#!/usr/bin/env python
# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import unittest

import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils


class TrackTests(unittest.TestCase, otio_test_utils.OTIOAssertions):

    def test_find_children(self):
        cl = otio.schema.Clip()
        tr = otio.schema.Track()
        tr.append(cl)
        result = tr.find_children(otio.schema.Clip)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0], cl)

    def test_find_children_search_range(self):
        range = otio.opentime.TimeRange(
            otio.opentime.RationalTime(0.0, 24.0),
            otio.opentime.RationalTime(24.0, 24.0))
        cl0 = otio.schema.Clip()
        cl0.source_range = range
        cl1 = otio.schema.Clip()
        cl1.source_range = range
        cl2 = otio.schema.Clip()
        cl2.source_range = range
        tr = otio.schema.Track()
        tr.append(cl0)
        tr.append(cl1)
        tr.append(cl2)
        result = tr.find_children(otio.schema.Clip, range)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0], cl0)

    def test_find_children_shallow_search(self):
        cl0 = otio.schema.Clip()
        cl1 = otio.schema.Clip()
        st = otio.schema.Stack()
        st.append(cl1)
        tr = otio.schema.Track()
        tr.append(cl0)
        tr.append(st)
        result = tr.find_children(otio.schema.Clip, shallow_search=True)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0], cl0)
        result = tr.find_children(otio.schema.Clip, shallow_search=False)
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], cl0)
        self.assertEqual(result[1], cl1)


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import unittest
import threading
import weakref

import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils


class MultithreadingTests(unittest.TestCase, otio_test_utils.OTIOAssertions):
    def test1(self):
        self.sc = otio.schema.SerializableCollection()
        child = otio.core.SerializableObject()
        child.extra = 17
        wc = weakref.ref(child)
        self.assertEqual(wc() is not None, True)
        self.sc.append(child)
        del child

        threads = []
        for i in range(5):
            t = threading.Thread(target=self.bash_retainers1)
            t.daemon = True
            t.start()
            threads.append(t)

        for t in threads:
            t.join()

        self.assertEqual(self.sc[0].extra, 17)
        self.sc.pop()
        self.assertEqual(wc() is None, True)

    def bash_retainers1(self):
        otio._otio._testing.bash_retainers1(self.sc)

    def test2(self):
        sc = otio.schema.SerializableCollection()
        child = otio.core.SerializableObject()
        sc.append(child)
        self.materialized = False

        self.sc = sc.clone()
        self.lock = threading.Lock()
        # self.sc[0] has not been given out to Python yet

        threads = []
        for i in range(5):
            t = threading.Thread(target=self.bash_retainers2)
            t.daemon = True
            t.start()
            threads.append(t)

        for t in threads:
            t.join()

        self.assertEqual(self.wc() is not None, True)
        self.assertEqual(self.wc().extra, 37)
        del self.sc
        self.assertEqual(self.wc() is None, True)

    def test3(self):
        t = threading.Thread(target=self.gil_scoping)
        t.daemon = True
        t.start()
        t.join()

    def test4(self):
        self.gil_scoping()

    def gil_scoping(self):
        otio._otio._testing.gil_scoping()

    def materialize(self):
        with self.lock:
            if not self.materialized:
                self.materialized = True
                child = self.sc[0]
                self.wc = weakref.ref(child)
                self.assertEqual(self.wc() is not None, True)
                child.extra = 37
                del child

    def bash_retainers2(self):
        otio._otio._testing.bash_retainers2(self.sc, self.materialize)


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""This file is here to support the test_schemadef_plugin unittest.
If you want to learn how to write your own SchemaDef plugin, please read:
https://opentimelineio.readthedocs.io/en/latest/tutorials/write-a-schemadef.html
"""

import opentimelineio as otio


@otio.core.register_type
class exampleSchemaDef(otio.core.SerializableObject):
    """Example of a SchemaDef plugin class for testing."""

    _serializable_label = "exampleSchemaDef.1"
    _name = "exampleSchemaDef"

    def __init__(
        self,
        exampleArg=None,
    ):
        otio.core.SerializableObject.__init__(self)
        self.exampleArg = exampleArg

    exampleArg = otio.core.serializable_field(
        "exampleArg",
        doc=(
            "example of an arg passed to the exampleSchemaDef"
        )
    )

    def __str__(self):
        return 'exampleSchemaDef({})'.format(
            repr(self.exampleArg)
        )

    def __repr__(self):
        return \
            'otio.schemadef.example_schemadef.exampleSchemaDef(exampleArg={})'.format(
                repr(self.exampleArg)
            )



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""Reusable utilities for tests."""

# import built-in modules
import os
import tempfile

# import local modules
import opentimelineio as otio
from tests import baseline_reader


MANIFEST_PATH = "adapter_plugin_manifest.plugin_manifest"


def create_manifest():
    """Create a temporary manifest."""
    full_baseline = baseline_reader.json_baseline_as_string(MANIFEST_PATH)

    temp_dir = tempfile.mkdtemp(prefix='test_otio_manifest')
    man_path = os.path.join(temp_dir, 'manifest')
    with open(man_path, 'w') as fo:
        fo.write(full_baseline)
    man = otio.plugins.manifest_from_file(man_path)
    man._update_plugin_source(baseline_reader.path_to_baseline(MANIFEST_PATH))
    return man


def remove_manifest(manifest):
    """Remove the manifest source files."""
    for file_path in manifest.source_files:
        # don't accidentally blow away python
        if not file_path.endswith('.py'):
            os.remove(file_path)



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""Unit tests for the JSON format OTIO Serializes to."""

import unittest
import json

import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils

# local to test dir
from tests import baseline_reader


class TestJsonFormat(unittest.TestCase, otio_test_utils.OTIOAssertions):

    def setUp(self):
        self.maxDiff = None

    def check_against_baseline(self, obj, testname):
        baseline = baseline_reader.json_baseline(testname)

        self.assertDictEqual(
            baseline_reader.json_from_string(
                otio.adapters.otio_json.write_to_string(obj)
            ),
            baseline
        )
        baseline_data = otio.adapters.otio_json.read_from_string(
            json.dumps(baseline)
        )
        if isinstance(baseline_data, dict):
            raise TypeError("did not deserialize correctly")

        self.assertJsonEqual(obj, baseline_data)

    def test_rationaltime(self):
        rt = otio.opentime.RationalTime()
        self.check_against_baseline(rt, "empty_rationaltime")

    def test_timerange(self):
        tr = otio.opentime.TimeRange()
        self.check_against_baseline(tr, "empty_timerange")

    def test_timetransform(self):
        tt = otio.opentime.TimeTransform()
        self.check_against_baseline(tt, "empty_timetransform")

    def test_track(self):
        st = otio.schema.Track(
            name="test_track",
            metadata={
                "comments": "adding some stuff to metadata to try out",
                "a number": 1.0
            }
        )
        self.check_against_baseline(st, "empty_track")

    def test_stack(self):
        st = otio.schema.Stack(
            name="tracks",
            metadata={
                "comments": "adding some stuff to metadata to try out",
                "a number": 1.0
            }
        )
        self.check_against_baseline(st, "empty_stack")

    def test_timeline(self):
        tl = otio.schema.Timeline(
            name="Example Timeline",
            metadata={
                "comments": "adding some stuff to metadata to try out",
                "a number": 1.0
            }
        )
        tl.tracks.metadata.update({
            "comments": "adding some stuff to metadata to try out",
            "a number": 1.0
        })
        self.check_against_baseline(tl, "empty_timeline")

    def test_clip(self):
        cl = otio.schema.Clip(
            name="test_clip",
            media_reference=otio.schema.MissingReference()
        )
        self.check_against_baseline(cl, "empty_clip")

    def test_gap(self):
        fl = otio.schema.Gap()
        self.check_against_baseline(fl, "empty_gap")

    def test_missing_reference(self):
        mr = otio.schema.MissingReference()
        self.check_against_baseline(mr, "empty_missingreference")

    def test_external_reference(self):
        mr = otio.schema.ExternalReference(target_url="foo.bar")
        self.check_against_baseline(mr, "empty_external_reference")

    def test_marker(self):
        mr = otio.schema.Marker()
        self.check_against_baseline(mr, "empty_marker")

    def test_transition(self):
        trx = otio.schema.Transition()
        self.check_against_baseline(trx, "empty_transition")

    def test_serializable_collection(self):
        tr = otio.schema.SerializableCollection(
            name="test",
            metadata={"foo": "bar"}
        )
        self.check_against_baseline(tr, "empty_serializable_collection")

    def test_generator_reference(self):
        trx = otio.schema.GeneratorReference()
        self.check_against_baseline(trx, "empty_generator_reference")


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""Transition class test harness."""

import unittest

import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils


class TransitionTests(unittest.TestCase, otio_test_utils.OTIOAssertions):
    def test_constructor(self):
        trx = otio.schema.Transition(
            name="AtoB",
            transition_type="SMPTE.Dissolve",
            metadata={
                "foo": "bar"
            }
        )
        self.assertEqual(trx.transition_type, "SMPTE.Dissolve")
        self.assertEqual(trx.name, "AtoB")
        self.assertEqual(trx.metadata, {"foo": "bar"})

    def test_serialize(self):
        trx = otio.schema.Transition(
            name="AtoB",
            transition_type="SMPTE.Dissolve",
            metadata={
                "foo": "bar"
            }
        )
        encoded = otio.adapters.otio_json.write_to_string(trx)
        decoded = otio.adapters.otio_json.read_from_string(encoded)
        self.assertIsOTIOEquivalentTo(trx, decoded)

    def test_stringify(self):
        trx = otio.schema.Transition("SMPTE.Dissolve")
        in_offset = otio.opentime.RationalTime(5, 24)
        out_offset = otio.opentime.RationalTime(1, 24)
        trx.in_offset = in_offset
        trx.out_offset = out_offset
        self.assertMultiLineEqual(
            str(trx),
            "Transition("
            '"{}", '
            '"{}", '
            '{}, '
            "{}, "
            "{}"
            ")".format(
                str(trx.name),
                str(trx.transition_type),
                str(trx.in_offset),
                str(trx.out_offset),
                str(trx.metadata),
            )
        )

        self.assertMultiLineEqual(
            repr(trx),
            "otio.schema.Transition("
            "name={}, "
            "transition_type={}, "
            "in_offset={}, "
            "out_offset={}, "
            "metadata={}"
            ")".format(
                repr(trx.name),
                repr(trx.transition_type),
                repr(trx.in_offset),
                repr(trx.out_offset),
                repr(trx.metadata),
            )
        )

    def test_setters(self):
        trx = otio.schema.Transition(
            name="AtoB",
            transition_type="SMPTE.Dissolve",
            metadata={
                "foo": "bar"
            }
        )
        self.assertEqual(trx.transition_type, "SMPTE.Dissolve")
        trx.transition_type = "EdgeWipe"
        self.assertEqual(trx.transition_type, "EdgeWipe")

    def test_parent_range(self):
        timeline = otio.schema.Timeline(
            tracks=[
                otio.schema.Track(
                    name="V1",
                    children=[
                        otio.schema.Clip(
                            name="A",
                            source_range=otio.opentime.TimeRange(
                                start_time=otio.opentime.RationalTime(
                                    value=1,
                                    rate=30
                                ),
                                duration=otio.opentime.RationalTime(
                                    value=50,
                                    rate=30
                                )
                            )
                        ),
                        otio.schema.Transition(
                            in_offset=otio.opentime.RationalTime(
                                value=7,
                                rate=30
                            ),
                            out_offset=otio.opentime.RationalTime(
                                value=10,
                                rate=30
                            ),
                        ),
                        otio.schema.Clip(
                            name="B",
                            source_range=otio.opentime.TimeRange(
                                start_time=otio.opentime.RationalTime(
                                    value=100,
                                    rate=30
                                ),
                                duration=otio.opentime.RationalTime(
                                    value=50,
                                    rate=30
                                )
                            )
                        ),
                    ]
                )
            ]
        )
        trx = timeline.tracks[0][1]
        time_range = otio.opentime.TimeRange(otio.opentime.RationalTime(43, 30),
                                             otio.opentime.RationalTime(17, 30))

        self.assertEqual(time_range,
                         trx.range_in_parent())

        self.assertEqual(time_range,
                         trx.trimmed_range_in_parent())

        trx = otio.schema.Transition(
            in_offset=otio.opentime.RationalTime(
                value=7,
                rate=30
            ),
            out_offset=otio.opentime.RationalTime(
                value=10,
                rate=30
            ),
        )

        with self.assertRaises(otio.exceptions.NotAChildError):
            trx.range_in_parent()

        with self.assertRaises(otio.exceptions.NotAChildError):
            trx.trimmed_range_in_parent()


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""Unit tests for the 'console' module."""

import unittest
import sys
import os
import subprocess
import sysconfig
import pathlib
import platform

import io

from tempfile import TemporaryDirectory  # noqa: F401
import tempfile

import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils
import opentimelineio.console as otio_console

SAMPLE_DATA_DIR = os.path.join(os.path.dirname(__file__), "sample_data")
MULTITRACK_PATH = os.path.join(SAMPLE_DATA_DIR, "multitrack.otio")
PREMIERE_EXAMPLE_PATH = os.path.join(SAMPLE_DATA_DIR, "premiere_example.otio")
SCREENING_EXAMPLE_PATH = os.path.join(SAMPLE_DATA_DIR, "screening_example.otio")
SIMPLE_CUT_PATH = os.path.join(SAMPLE_DATA_DIR, "simple_cut.otio")
TRANSITION_PATH = os.path.join(SAMPLE_DATA_DIR, "transition.otio")


def CreateShelloutTest(cl):
    if os.environ.get("OTIO_DISABLE_SHELLOUT_TESTS"):
        newSuite = None
    else:
        class newSuite(cl):
            SHELL_OUT = True
        newSuite.__name__ = cl.__name__ + "_on_shell"

    return newSuite


class ConsoleTester(otio_test_utils.OTIOAssertions):
    """ Base class for running console tests both by directly calling main() and
    by shelling out.
    """

    SHELL_OUT = False

    def setUp(self):
        self.saved_args = sys.argv
        self.old_stdout = sys.stdout
        self.old_stderr = sys.stderr
        sys.stdout = io.StringIO()
        sys.stderr = io.StringIO()

    def run_test(self):
        if self.SHELL_OUT:
            # make sure its on the path
            console_script = os.path.join(sysconfig.get_path('scripts'), sys.argv[0])
            if platform.system() == 'Windows':
                console_script += '.exe'

            if not os.path.exists(console_script):
                self.fail(
                    "Could not find '{}'.  Tests that explicitly shell"
                    " out can be disabled by setting the environment variable "
                    "OTIO_DISABLE_SHELLOUT_TESTS.".format(console_script)
                )

            # actually run the test (sys.argv is already populated correctly)
            proc = subprocess.Popen(
                sys.argv,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )
            stdout, stderr = proc.communicate()

            # XXX 2.7 vs 3.xx bug
            if type(stdout) is not str:
                stdout = stdout.decode("utf-8")
                stderr = stderr.decode("utf-8")

            sys.stdout.write(stdout)
            sys.stderr.write(stderr)

            if proc.returncode != 0:
                raise SystemExit()
        else:
            self.test_module.main()

        # pre-fetch these strings for easy access
        stdout = sys.stdout.getvalue()
        stderr = sys.stderr.getvalue()

        if platform.system() == 'Windows':
            # Normalize line-endings for assertEqual(expected, actual)
            stdout = stdout.replace('\r\n', '\n')
            stderr = stderr.replace('\r\n', '\n')

        return stdout, stderr

    def tearDown(self):
        sys.stdout = self.old_stdout
        sys.stderr = self.old_stderr
        sys.argv = self.saved_args


class OTIOStatTest(ConsoleTester, unittest.TestCase):
    test_module = otio_console.otiostat

    def test_basic(self):
        sys.argv = ['otiostat', SCREENING_EXAMPLE_PATH]
        self.run_test()
        self.assertIn("top level object: Timeline.1", sys.stdout.getvalue())


OTIOStatTest_ShellOut = CreateShelloutTest(OTIOStatTest)


class OTIOCatTests(ConsoleTester, unittest.TestCase):
    test_module = otio_console.otiocat

    def test_basic(self):
        sys.argv = ['otiocat', SCREENING_EXAMPLE_PATH]
        self.run_test()
        self.assertIn('"name": "Example_Screening.01",', sys.stdout.getvalue())

    def test_no_media_linker(self):
        sys.argv = ['otiocat', SCREENING_EXAMPLE_PATH, "-m", "none"]
        self.run_test()
        self.assertIn('"name": "Example_Screening.01",', sys.stdout.getvalue())

    def test_input_argument_error(self):
        sys.argv = [
            'otiocat',
            SCREENING_EXAMPLE_PATH,
            "-a", "foobar",
        ]

        with self.assertRaises(SystemExit):
            self.run_test()

        # read results back in
        self.assertIn('error: adapter', sys.stderr.getvalue())

    def test_media_linker_argument_error(self):
        sys.argv = [
            'otiocat',
            SCREENING_EXAMPLE_PATH,
            "-M", "foobar",
        ]

        with self.assertRaises(SystemExit):
            self.run_test()

        # read results back in
        self.assertIn('error: media linker', sys.stderr.getvalue())


OTIOCatTests_OnShell = CreateShelloutTest(OTIOCatTests)


class OTIOConvertTests(ConsoleTester, unittest.TestCase):
    test_module = otio_console.otioconvert

    def test_basic(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_file = os.path.join(temp_dir, "test_basic.otio")
            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                '--tracks', '0'
            ]
            self.run_test()

            # read results back in
            with open(temp_file) as fi:
                self.assertIn('"name": "Example_Screening.01",', fi.read())

    def test_begin_end(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_file = os.path.join(temp_dir, "test_begin_end.otio")

            # begin needs to be a,b
            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                "--begin", "foobar"
            ]
            with self.assertRaises(SystemExit):
                self.run_test()

            # end requires begin
            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                "--end", "foobar"
            ]
            with self.assertRaises(SystemExit):
                self.run_test()

            # prune everything
            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                "--begin", "0,24",
                "--end", "0,24",
            ]
            otio_console.otioconvert.main()

            # check that begin/end "," parsing is checked
            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                "--begin", "0",
                "--end", "0,24",
            ]
            with self.assertRaises(SystemExit):
                self.run_test()

            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                "--begin", "0,24",
                "--end", "0",
            ]
            with self.assertRaises(SystemExit):
                self.run_test()

            result = otio.adapters.read_from_file(temp_file, "otio_json")
            self.assertEqual(len(result.tracks[0]), 0)
            self.assertEqual(result.name, "Example_Screening.01")

    def test_input_argument_error(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_file = os.path.join(temp_dir, "test_input_argument_error.otio")

            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                "-a", "foobar",
            ]

            with self.assertRaises(SystemExit):
                self.run_test()

            # read results back in
            self.assertIn('error: input adapter', sys.stderr.getvalue())

    def test_output_argument_error(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_file = os.path.join(temp_dir, "test_output_argument_error.otio")

            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                "-A", "foobar",
            ]

            with self.assertRaises(SystemExit):
                self.run_test()

            # read results back in
            self.assertIn('error: output adapter', sys.stderr.getvalue())

    def test_media_linker_argument_error(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_file = os.path.join(temp_dir, "test_media_linker_argument_error.otio")

            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                "-m", "fake_linker",
                "-M", "somestring=foobar",
                "-M", "foobar",
            ]

            with self.assertRaises(SystemExit):
                self.run_test()

            # read results back in
            self.assertIn('error: media linker', sys.stderr.getvalue())


OTIOConvertTests_OnShell = CreateShelloutTest(OTIOConvertTests)


class OTIOPlugInfoTest(ConsoleTester, unittest.TestCase):
    test_module = otio_console.otiopluginfo

    def test_basic(self):
        sys.argv = ['otiopluginfo']
        self.run_test()
        self.assertIn("Manifests loaded:", sys.stdout.getvalue())


OTIOPlugInfoTest_ShellOut = CreateShelloutTest(OTIOStatTest)


class OTIOToolTest(ConsoleTester, unittest.TestCase):
    test_module = otio_console.otiotool

    def test_list_tracks(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--list-tracks'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: OTIO TEST - multitrack.Exported.01
TRACK: Sequence (Video)
TRACK: Sequence 2 (Video)
TRACK: Sequence 3 (Video)
""", out)

    def test_list_clips(self):
        sys.argv = [
            'otiotool',
            '-i', SCREENING_EXAMPLE_PATH,
            '--list-clips'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: Example_Screening.01
  CLIP: ZZ100_501 (LAY3)
  CLIP: ZZ100_502A (LAY3)
  CLIP: ZZ100_503A (LAY1)
  CLIP: ZZ100_504C (LAY1)
  CLIP: ZZ100_504B (LAY1)
  CLIP: ZZ100_507C (LAY2)
  CLIP: ZZ100_508 (LAY2)
  CLIP: ZZ100_510 (LAY1)
  CLIP: ZZ100_510B (LAY1)
""", out)

    def test_list_markers(self):
        sys.argv = [
            'otiotool',
            '-i', PREMIERE_EXAMPLE_PATH,
            '--list-markers'
        ]
        out, err = self.run_test()
        self.assertEqual(
            ("TIMELINE: sc01_sh010_layerA\n"
             "  MARKER: global: 00:00:03:23 local: 00:00:03:23 duration: 0.0 color: RED name: My MArker 1\n"  # noqa: E501 line too long
             "  MARKER: global: 00:00:16:12 local: 00:00:16:12 duration: 0.0 color: RED name: dsf\n"  # noqa: E501 line too long
             "  MARKER: global: 00:00:09:28 local: 00:00:09:28 duration: 0.0 color: RED name: \n"  # noqa: E501 line too long
             "  MARKER: global: 00:00:13:05 local: 00:00:02:13 duration: 0.0 color: RED name: \n"),  # noqa: E501 line too long
            out)

    def test_list_tracks_and_clips(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--list-tracks',
            '--list-clips'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: OTIO TEST - multitrack.Exported.01
TRACK: Sequence (Video)
  CLIP: tech.fux (loop)-HD.mp4
  CLIP: out-b (loop)-HD.mp4
  CLIP: brokchrd (loop)-HD.mp4
TRACK: Sequence 2 (Video)
  CLIP: t-hawk (loop)-HD.mp4
TRACK: Sequence 3 (Video)
  CLIP: KOLL-HD.mp4
""", out)

    def test_list_tracks_and_clips_and_media(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--list-tracks',
            '--list-clips',
            '--list-media'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: OTIO TEST - multitrack.Exported.01
TRACK: Sequence (Video)
  CLIP: tech.fux (loop)-HD.mp4
    MEDIA: None
  CLIP: out-b (loop)-HD.mp4
    MEDIA: None
  CLIP: brokchrd (loop)-HD.mp4
    MEDIA: None
TRACK: Sequence 2 (Video)
  CLIP: t-hawk (loop)-HD.mp4
    MEDIA: None
TRACK: Sequence 3 (Video)
  CLIP: KOLL-HD.mp4
    MEDIA: None
""", out)

    def test_list_tracks_and_clips_and_media_and_markers(self):
        sys.argv = [
            'otiotool',
            '-i', PREMIERE_EXAMPLE_PATH,
            '--list-tracks',
            '--list-clips',
            '--list-media',
            '--list-markers'
        ]
        out, err = self.run_test()
        self.assertEqual(
            ("TIMELINE: sc01_sh010_layerA\n"
             "  MARKER: global: 00:00:03:23 local: 00:00:03:23 duration: 0.0 color: RED name: My MArker 1\n"  # noqa E501 line too long
             "  MARKER: global: 00:00:16:12 local: 00:00:16:12 duration: 0.0 color: RED name: dsf\n"  # noqa E501 line too long
             "  MARKER: global: 00:00:09:28 local: 00:00:09:28 duration: 0.0 color: RED name: \n"  # noqa E501 line too long
             "TRACK:  (Video)\n"
             "  CLIP: sc01_sh010_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh010_anim.mov\n"
             "TRACK:  (Video)\n"
             "  CLIP: sc01_sh010_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh010_anim.mov\n"
             "  CLIP: sc01_sh020_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh020_anim.mov\n"
             "  CLIP: sc01_sh030_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh030_anim.mov\n"
             "  MARKER: global: 00:00:13:05 local: 00:00:02:13 duration: 0.0 color: RED name: \n"  # noqa E501 line too long
             "TRACK:  (Video)\n"
             "  CLIP: test_title\n"
             "    MEDIA: None\n"
             "TRACK:  (Video)\n"
             "  CLIP: sc01_master_layerA_sh030_temp.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_master_layerA_sh030_temp.mov\n"  # noqa E501 line too long
             "  CLIP: sc01_sh010_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh010_anim.mov\n"
             "TRACK:  (Audio)\n"
             "  CLIP: sc01_sh010_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh010_anim.mov\n"
             "  CLIP: sc01_sh010_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh010_anim.mov\n"
             "TRACK:  (Audio)\n"
             "  CLIP: sc01_placeholder.wav\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_placeholder.wav\n"
             "TRACK:  (Audio)\n"
             "  CLIP: track_08.wav\n"
             "    MEDIA: file://localhost/D%3a/media/track_08.wav\n"
             "TRACK:  (Audio)\n"
             "  CLIP: sc01_master_layerA_sh030_temp.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_master_layerA_sh030_temp.mov\n"  # noqa E501 line too long
             "  CLIP: sc01_sh010_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh010_anim.mov\n"),
            out)

    def test_verify_media(self):
        sys.argv = [
            'otiotool',
            '-i', PREMIERE_EXAMPLE_PATH,
            '--list-tracks',
            '--list-clips',
            '--list-media',
            '--verify-media'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: sc01_sh010_layerA
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh010_anim.mov
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh010_anim.mov
  CLIP: sc01_sh020_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh020_anim.mov
  CLIP: sc01_sh030_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh030_anim.mov
TRACK:  (Video)
  CLIP: test_title
    MEDIA: None
TRACK:  (Video)
  CLIP: sc01_master_layerA_sh030_temp.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_sh010_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh010_anim.mov
  CLIP: sc01_sh010_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_placeholder.wav
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_placeholder.wav
TRACK:  (Audio)
  CLIP: track_08.wav
    MEDIA NOT FOUND: file://localhost/D%3a/media/track_08.wav
TRACK:  (Audio)
  CLIP: sc01_master_layerA_sh030_temp.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh010_anim.mov
""", out)

    def test_video_only(self):
        sys.argv = [
            'otiotool',
            '-i', PREMIERE_EXAMPLE_PATH,
            '--video-only',
            '--list-clips'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: sc01_sh010_layerA
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh020_anim.mov
  CLIP: sc01_sh030_anim.mov
  CLIP: test_title
  CLIP: sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
""", out)

    def test_audio_only(self):
        sys.argv = [
            'otiotool',
            '-i', PREMIERE_EXAMPLE_PATH,
            '--audio-only',
            '--list-clips'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: sc01_sh010_layerA
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_placeholder.wav
  CLIP: track_08.wav
  CLIP: sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
""", out)

    def test_only_tracks_with_name(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--only-tracks-with-name', 'Sequence 3',
            '--list-clips'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: OTIO TEST - multitrack.Exported.01
  CLIP: KOLL-HD.mp4
""", out)

    def test_only_tracks_with_index(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--only-tracks-with-index', '3',
            '--list-clips'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: OTIO TEST - multitrack.Exported.01
  CLIP: KOLL-HD.mp4
""", out)

    def test_only_tracks_with_index2(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--only-tracks-with-index', '2', '3',
            '--list-clips'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: OTIO TEST - multitrack.Exported.01
  CLIP: t-hawk (loop)-HD.mp4
  CLIP: KOLL-HD.mp4
""", out)

    def test_only_clips_with_name(self):
        sys.argv = [
            'otiotool',
            '-i', PREMIERE_EXAMPLE_PATH,
            '--list-clips',
            '--list-tracks',
            '--only-clips-with-name', 'sc01_sh010_anim.mov'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: sc01_sh010_layerA
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
TRACK:  (Video)
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
TRACK:  (Audio)
TRACK:  (Audio)
  CLIP: sc01_sh010_anim.mov
""", out)

    def test_only_clips_with_regex(self):
        sys.argv = [
            'otiotool',
            '-i', PREMIERE_EXAMPLE_PATH,
            '--list-clips',
            '--list-tracks',
            '--only-clips-with-name-regex', 'anim'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: sc01_sh010_layerA
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh020_anim.mov
  CLIP: sc01_sh030_anim.mov
TRACK:  (Video)
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
TRACK:  (Audio)
TRACK:  (Audio)
  CLIP: sc01_sh010_anim.mov
""", out)

    def test_remote_transition(self):
        sys.argv = [
            'otiotool',
            '-i', TRANSITION_PATH,
            '-o', '-',
            '--remove-transitions'
        ]
        out, err = self.run_test()
        self.assertNotIn('"OTIO_SCHEMA": "Transition.', out)

    def test_trim(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--trim', '20', '40',
            '--list-clips',
            '--inspect', 't-hawk'
        ]
        out, err = self.run_test()
        self.assertEqual(
            ("TIMELINE: OTIO TEST - multitrack.Exported.01\n"
             "  ITEM: t-hawk (loop)-HD.mp4 (<class 'opentimelineio._otio.Clip'>)\n"  # noqa E501 line too long
             "    source_range: TimeRange(RationalTime(0, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "    trimmed_range: TimeRange(RationalTime(0, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "    visible_range: TimeRange(RationalTime(0, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "    range_in_parent: TimeRange(RationalTime(2, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "    trimmed range in timeline: TimeRange(RationalTime(2, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "    visible range in timeline: TimeRange(RationalTime(2, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "    range in Sequence 2 (<class 'opentimelineio._otio.Track'>): TimeRange(RationalTime(2, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "    range in NestedScope (<class 'opentimelineio._otio.Stack'>): TimeRange(RationalTime(2, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "TIMELINE: OTIO TEST - multitrack.Exported.01\n"
             "  CLIP: tech.fux (loop)-HD.mp4\n"
             "  CLIP: out-b (loop)-HD.mp4\n"
             "  CLIP: t-hawk (loop)-HD.mp4\n"),
            out)

    def test_flatten(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--flatten', 'video',
            '--list-clips',
            '--list-tracks',
            '--inspect', 'out-b'
        ]
        out, err = self.run_test()
        self.assertEqual(
            ("TIMELINE: OTIO TEST - multitrack.Exported.01\n"
             "  ITEM: out-b (loop)-HD.mp4 (<class 'opentimelineio._otio.Clip'>)\n"  # noqa E501 line too long
             "    source_range: TimeRange(RationalTime(159, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    trimmed_range: TimeRange(RationalTime(159, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    visible_range: TimeRange(RationalTime(159, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    range_in_parent: TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    trimmed range in timeline: TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    visible range in timeline: TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    range in Flattened (<class 'opentimelineio._otio.Track'>): TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    range in NestedScope (<class 'opentimelineio._otio.Stack'>): TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "TIMELINE: OTIO TEST - multitrack.Exported.01\n"
             "TRACK: Flattened (Video)\n"
             "  CLIP: tech.fux (loop)-HD.mp4\n"
             "  CLIP: t-hawk (loop)-HD.mp4\n"
             "  CLIP: out-b (loop)-HD.mp4\n"
             "  CLIP: KOLL-HD.mp4\n"
             "  CLIP: brokchrd (loop)-HD.mp4\n"),
            out)

    def test_keep_flattened_tracks(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--flatten', 'video',
            '--keep-flattened-tracks',
            '--list-clips',
            '--list-tracks',
            '--inspect', 'out-b'
        ]
        out, err = self.run_test()
        self.assertEqual(
            ("TIMELINE: OTIO TEST - multitrack.Exported.01\n"
             "  ITEM: out-b (loop)-HD.mp4 (<class 'opentimelineio._otio.Clip'>)\n"  # noqa E501 line too long
             "    source_range: TimeRange(RationalTime(0, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "    trimmed_range: TimeRange(RationalTime(0, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "    visible_range: TimeRange(RationalTime(0, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "    range_in_parent: TimeRange(RationalTime(803, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "    trimmed range in timeline: TimeRange(RationalTime(803, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "    visible range in timeline: TimeRange(RationalTime(803, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "    range in Sequence (<class 'opentimelineio._otio.Track'>): TimeRange(RationalTime(803, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "    range in NestedScope (<class 'opentimelineio._otio.Stack'>): TimeRange(RationalTime(803, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "  ITEM: out-b (loop)-HD.mp4 (<class 'opentimelineio._otio.Clip'>)\n"  # noqa E501 line too long
             "    source_range: TimeRange(RationalTime(159, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    trimmed_range: TimeRange(RationalTime(159, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    visible_range: TimeRange(RationalTime(159, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    range_in_parent: TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    trimmed range in timeline: TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    visible range in timeline: TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    range in Flattened (<class 'opentimelineio._otio.Track'>): TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    range in NestedScope (<class 'opentimelineio._otio.Stack'>): TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "TIMELINE: OTIO TEST - multitrack.Exported.01\n"
             "TRACK: Sequence (Video)\n"
             "  CLIP: tech.fux (loop)-HD.mp4\n"
             "  CLIP: out-b (loop)-HD.mp4\n"
             "  CLIP: brokchrd (loop)-HD.mp4\n"
             "TRACK: Sequence 2 (Video)\n"
             "  CLIP: t-hawk (loop)-HD.mp4\n"
             "TRACK: Sequence 3 (Video)\n"
             "  CLIP: KOLL-HD.mp4\n"
             "TRACK: Flattened (Video)\n"
             "  CLIP: tech.fux (loop)-HD.mp4\n"
             "  CLIP: t-hawk (loop)-HD.mp4\n"
             "  CLIP: out-b (loop)-HD.mp4\n"
             "  CLIP: KOLL-HD.mp4\n"
             "  CLIP: brokchrd (loop)-HD.mp4\n"),
            out)

    def test_stack(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH, PREMIERE_EXAMPLE_PATH,
            '--stack',
            '--list-clips',
            '--list-tracks',
            '--stats'
        ]
        out, err = self.run_test()
        self.maxDiff = None
        self.assertEqual("""Name: Stacked 2 Timelines
Start:    00:00:00:00
End:      00:02:16:18
Duration: 00:02:16:18
TIMELINE: Stacked 2 Timelines
TRACK: Sequence (Video)
  CLIP: tech.fux (loop)-HD.mp4
  CLIP: out-b (loop)-HD.mp4
  CLIP: brokchrd (loop)-HD.mp4
TRACK: Sequence 2 (Video)
  CLIP: t-hawk (loop)-HD.mp4
TRACK: Sequence 3 (Video)
  CLIP: KOLL-HD.mp4
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh020_anim.mov
  CLIP: sc01_sh030_anim.mov
TRACK:  (Video)
  CLIP: test_title
TRACK:  (Video)
  CLIP: sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_placeholder.wav
TRACK:  (Audio)
  CLIP: track_08.wav
TRACK:  (Audio)
  CLIP: sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
""", out)

    def test_concat(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH, PREMIERE_EXAMPLE_PATH,
            '--concat',
            '--list-clips',
            '--list-tracks',
            '--stats'
        ]
        out, err = self.run_test()
        self.maxDiff = None
        self.assertEqual("""Name: Concatenated 2 Timelines
Start:    00:00:00:00
End:      00:02:59:03
Duration: 00:02:59:03
TIMELINE: Concatenated 2 Timelines
TRACK:  (Video)
TRACK: Sequence (Video)
  CLIP: tech.fux (loop)-HD.mp4
  CLIP: out-b (loop)-HD.mp4
  CLIP: brokchrd (loop)-HD.mp4
TRACK: Sequence 2 (Video)
  CLIP: t-hawk (loop)-HD.mp4
TRACK: Sequence 3 (Video)
  CLIP: KOLL-HD.mp4
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh020_anim.mov
  CLIP: sc01_sh030_anim.mov
TRACK:  (Video)
  CLIP: test_title
TRACK:  (Video)
  CLIP: sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_placeholder.wav
TRACK:  (Audio)
  CLIP: track_08.wav
TRACK:  (Audio)
  CLIP: sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
""", out)

    def test_redact(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--redact',
            '--list-clips',
            '--list-tracks'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: Timeline #1
TRACK: Track #1 (Video)
  CLIP: Clip #1
  CLIP: Clip #2
  CLIP: Clip #3
TRACK: Track #2 (Video)
  CLIP: Clip #4
TRACK: Track #3 (Video)
  CLIP: Clip #5
""", out)

    def test_stats(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--stats'
        ]
        out, err = self.run_test()
        self.assertEqual("""Name: OTIO TEST - multitrack.Exported.01
Start:    00:00:00:00
End:      00:02:16:18
Duration: 00:02:16:18
""", out)

    def test_inspect(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--inspect', 'KOLL'
        ]
        out, err = self.run_test()
        self.assertEqual(
            ("TIMELINE: OTIO TEST - multitrack.Exported.01\n"
             "  ITEM: KOLL-HD.mp4 (<class 'opentimelineio._otio.Clip'>)\n"
             "    source_range: TimeRange(RationalTime(0, 24), RationalTime(640, 24))\n"  # noqa E501 line too long
             "    trimmed_range: TimeRange(RationalTime(0, 24), RationalTime(640, 24))\n"  # noqa E501 line too long
             "    visible_range: TimeRange(RationalTime(0, 24), RationalTime(640, 24))\n"  # noqa E501 line too long
             "    range_in_parent: TimeRange(RationalTime(1198, 24), RationalTime(640, 24))\n"  # noqa E501 line too long
             "    trimmed range in timeline: TimeRange(RationalTime(1198, 24), RationalTime(640, 24))\n"  # noqa E501 line too long
             "    visible range in timeline: TimeRange(RationalTime(1198, 24), RationalTime(640, 24))\n"  # noqa E501 line too long
             "    range in Sequence 3 (<class 'opentimelineio._otio.Track'>): TimeRange(RationalTime(1198, 24), RationalTime(640, 24))\n"  # noqa E501 line too long
             "    range in NestedScope (<class 'opentimelineio._otio.Stack'>): TimeRange(RationalTime(1198, 24), RationalTime(640, 24))\n"),  # noqa E501 line too long
            out)

    def test_relink(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_file1 = os.path.join(temp_dir, "Clip-001.empty")
            temp_file2 = os.path.join(temp_dir, "Clip-003.empty")
            open(temp_file1, "w").write("A")
            open(temp_file2, "w").write("B")

            temp_url = pathlib.Path(temp_dir).as_uri()

            sys.argv = [
                'otiotool',
                '-i', SIMPLE_CUT_PATH,
                '--relink-by-name', temp_dir,
                '--list-media'
            ]
            out, err = self.run_test()
            self.assertIn(
                ("TIMELINE: Figure 1 - Simple Cut List\n"
                 f"    MEDIA: {temp_url}/Clip-001.empty\n"
                 "    MEDIA: file:///folder/wind-up.mov\n"
                 f"    MEDIA: {temp_url}/Clip-003.empty\n"
                 "    MEDIA: file:///folder/credits.mov\n"),
                out)


OTIOToolTest_ShellOut = CreateShelloutTest(OTIOToolTest)


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

""" Unit tests of functions that convert between file paths and urls. """

import unittest
import os

import opentimelineio as otio

SAMPLE_DATA_DIR = os.path.join(os.path.dirname(__file__), "sample_data")
SCREENING_EXAMPLE_PATH = os.path.join(SAMPLE_DATA_DIR, "screening_example.otio")
MEDIA_EXAMPLE_PATH_REL = os.path.relpath(
    os.path.join(
        os.path.dirname(__file__),
        "..",  # root
        "docs",
        "_static",
        "OpenTimelineIO@3xDark.png"
    )
)
MEDIA_EXAMPLE_PATH_URL_REL = otio.url_utils.url_from_filepath(
    MEDIA_EXAMPLE_PATH_REL
)
MEDIA_EXAMPLE_PATH_ABS = os.path.abspath(
    MEDIA_EXAMPLE_PATH_REL.replace(
        "3xDark",
        "3xLight"
    )
)
MEDIA_EXAMPLE_PATH_URL_ABS = otio.url_utils.url_from_filepath(
    MEDIA_EXAMPLE_PATH_ABS
)

WINDOWS_ENCODED_URL = "file://host/S%3a/path/file.ext"
WINDOWS_DRIVE_URL = "file://S:/path/file.ext"
WINDOWS_DRIVE_PATH = "S:/path/file.ext"

WINDOWS_ENCODED_UNC_URL = "file://unc/path/sub%20dir/file.ext"
WINDOWS_UNC_URL = "file://unc/path/sub dir/file.ext"
WINDOWS_UNC_PATH = "//unc/path/sub dir/file.ext"

POSIX_LOCALHOST_URL = "file://localhost/path/sub dir/file.ext"
POSIX_ENCODED_URL = "file:///path/sub%20dir/file.ext"
POSIX_URL = "file:///path/sub dir/file.ext"
POSIX_PATH = "/path/sub dir/file.ext"


class TestConversions(unittest.TestCase):
    def test_roundtrip_abs(self):
        self.assertTrue(MEDIA_EXAMPLE_PATH_URL_ABS.startswith("file://"))
        full_path = os.path.abspath(
            otio.url_utils.filepath_from_url(MEDIA_EXAMPLE_PATH_URL_ABS)
        )

        # should have reconstructed it by this point
        self.assertEqual(full_path, MEDIA_EXAMPLE_PATH_ABS)

    def test_roundtrip_rel(self):
        self.assertFalse(MEDIA_EXAMPLE_PATH_URL_REL.startswith("file://"))

        result = otio.url_utils.filepath_from_url(MEDIA_EXAMPLE_PATH_URL_REL)

        # should have reconstructed it by this point
        self.assertEqual(os.path.normpath(result), MEDIA_EXAMPLE_PATH_REL)

    def test_windows_urls(self):
        for url in (WINDOWS_ENCODED_URL, WINDOWS_DRIVE_URL):
            processed_url = otio.url_utils.filepath_from_url(url)
            self.assertEqual(processed_url, WINDOWS_DRIVE_PATH)

    def test_windows_unc_urls(self):
        for url in (WINDOWS_ENCODED_UNC_URL, WINDOWS_UNC_URL):
            processed_url = otio.url_utils.filepath_from_url(url)
            self.assertEqual(processed_url, WINDOWS_UNC_PATH)

    def test_posix_urls(self):
        for url in (POSIX_ENCODED_URL, POSIX_URL, POSIX_LOCALHOST_URL):
            processed_url = otio.url_utils.filepath_from_url(url)
            self.assertEqual(processed_url, POSIX_PATH)


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import unittest

import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils


class MarkerTest(unittest.TestCase, otio_test_utils.OTIOAssertions):

    def test_cons(self):
        tr = otio.opentime.TimeRange(
            otio.opentime.RationalTime(5, 24),
            otio.opentime.RationalTime(10, 24)
        )
        m = otio.schema.Marker(
            name="marker_1",
            marked_range=tr,
            color=otio.schema.MarkerColor.GREEN,
            metadata={'foo': 'bar'}
        )
        self.assertEqual(m.name, 'marker_1')
        self.assertEqual(m.metadata['foo'], 'bar')
        self.assertEqual(m.marked_range, tr)
        self.assertEqual(m.color, otio.schema.MarkerColor.GREEN)

        encoded = otio.adapters.otio_json.write_to_string(m)
        decoded = otio.adapters.otio_json.read_from_string(encoded)
        self.assertIsOTIOEquivalentTo(m, decoded)

    def test_upgrade(self):
        src = """
        {
            "OTIO_SCHEMA" : "Marker.1",
            "metadata" : {},
            "name" : null,
            "range" : {
                "OTIO_SCHEMA" : "TimeRange.1",
                "start_time" : {
                    "OTIO_SCHEMA" : "RationalTime.1",
                    "rate" : 5,
                    "value" : 0
                },
                "duration" : {
                    "OTIO_SCHEMA" : "RationalTime.1",
                    "rate" : 5,
                    "value" : 0
                }
            }

        }
        """
        marker = otio.adapters.read_from_string(src, "otio_json")
        self.assertEqual(
            marker.marked_range,
            otio.opentime.TimeRange(
                otio.opentime.RationalTime(0, 5),
                otio.opentime.RationalTime(0, 5),
            )
        )

    def test_equality(self):
        m = otio.schema.Marker()
        bo = otio.core.Item()

        self.assertNotEqual(m, bo)
        self.assertNotEqual(bo, m)

    def test_repr(self):
        tr = otio.opentime.TimeRange(
            otio.opentime.RationalTime(5, 24),
            otio.opentime.RationalTime(10, 24)
        )
        m = otio.schema.Marker(
            name="marker_1",
            marked_range=tr,
            color=otio.schema.MarkerColor.GREEN,
            metadata={'foo': 'bar'}
        )

        expected = (
            "otio.schema.Marker(name='marker_1', "
            "marked_range={}, metadata={})".format(
                repr(tr), repr(m.metadata)
            )
        )

        self.assertEqual(repr(m), expected)

    def test_comment(self):
        src = """
        {
            "OTIO_SCHEMA" : "Marker.1",
            "metadata" : {},
            "name" : null,
            "comment": "foo bar",
            "range" : {
                "OTIO_SCHEMA" : "TimeRange.1",
                "start_time" : {
                    "OTIO_SCHEMA" : "RationalTime.1",
                    "rate" : 5,
                    "value" : 0
                },
                "duration" : {
                    "OTIO_SCHEMA" : "RationalTime.1",
                    "rate" : 5,
                    "value" : 0
                }
            }

        }
        """
        marker = otio.adapters.read_from_string(src, "otio_json")
        self.assertEqual(marker.comment, "foo bar")

        tr = otio.opentime.TimeRange(
            otio.opentime.RationalTime(5, 24),
            otio.opentime.RationalTime(10, 24)
        )
        m = otio.schema.Marker(
            name="marker_1",
            marked_range=tr,
            metadata={'foo': 'bar'},
            comment="foo bar2")
        self.assertEqual(m.comment, "foo bar2")

        encoded = otio.adapters.otio_json.write_to_string(m)
        decoded = otio.adapters.otio_json.read_from_string(encoded)
        self.assertEqual(decoded.comment, "foo bar2")

    def test_str(self):
        tr = otio.opentime.TimeRange(
            otio.opentime.RationalTime(5, 24),
            otio.opentime.RationalTime(10, 24)
        )
        m = otio.schema.Marker(
            name="marker_1",
            marked_range=tr,
            color=otio.schema.MarkerColor.GREEN,
            metadata={'foo': 'bar'}
        )

        expected = 'Marker(marker_1, {}, {})'.format(
            str(tr), str(m.metadata)
        )

        self.assertEqual(str(m), expected)


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import copy
import unittest

import opentimelineio._otio
import opentimelineio.core._core_utils


class AnyDictionaryTests(unittest.TestCase):
    def test_main(self):
        d = opentimelineio.core._core_utils.AnyDictionary()
        d['a'] = 1

        self.assertTrue('a' in d)
        self.assertFalse('asdasdasd' in d)

        self.assertEqual(len(d), 1)
        self.assertEqual(d['a'], 1)  # New key

        with self.assertRaisesRegex(KeyError, "'non-existent'"):
            d['non-existent']

        # TODO: Test different type of values to exercise the any_to_py function?

        d['a'] = 'newvalue'
        self.assertEqual(d['a'], 'newvalue')

        self.assertTrue('a' in d)  # Test __contains__
        self.assertFalse('b' in d)

        with self.assertRaises(TypeError):
            d[1]  # AnyDictionary.__getitem__ only supports strings

        del d['a']
        self.assertEqual(dict(d), {})
        with self.assertRaisesRegex(KeyError, "'non-existent'"):
            del d['non-existent']

        for key in iter(d):  # Test AnyDictionaryProxy.Iterator.iter
            self.assertTrue(key)

        class CustomClass(object):
            pass

        with self.assertRaises(TypeError):
            d['custom'] = CustomClass()

        with self.assertRaises(ValueError):
            # Integer bigger than C++ int64_t can accept.
            d['super big int'] = 9223372036854775808

        with self.assertRaises(ValueError):
            # Integer smaller than C++ int64_t can accept.
            d['super big int'] = -9223372036854775809

    def test_raise_on_mutation_during_iter(self):
        d = opentimelineio.core._core_utils.AnyDictionary()
        d['a'] = 'test'
        d['b'] = 'asdasda'

        with self.assertRaisesRegex(ValueError, "container mutated during iteration"):
            for key in d:
                del d['b']

    def test_raises_if_ref_destroyed(self):
        d1 = opentimelineio.core._core_utils.AnyDictionary()
        opentimelineio._otio._testing.test_AnyDictionary_destroy(d1)

        with self.assertRaisesRegex(ValueError, r"Underlying C\+\+ AnyDictionary has been destroyed"):  # noqa
            d1['asd']

        d2 = opentimelineio.core._core_utils.AnyDictionary()
        opentimelineio._otio._testing.test_AnyDictionary_destroy(d2)

        with self.assertRaisesRegex(ValueError, r"Underlying C\+\+ AnyDictionary has been destroyed"):  # noqa
            d2['asd'] = 'asd'

        d3 = opentimelineio.core._core_utils.AnyDictionary()
        opentimelineio._otio._testing.test_AnyDictionary_destroy(d3)

        with self.assertRaisesRegex(ValueError, r"Underlying C\+\+ AnyDictionary has been destroyed"):  # noqa
            del d3['asd']

        d4 = opentimelineio.core._core_utils.AnyDictionary()
        d4['asd'] = 1
        it = iter(d4)
        opentimelineio._otio._testing.test_AnyDictionary_destroy(d4)
        with self.assertRaisesRegex(ValueError, r"Underlying C\+\+ AnyDictionary has been destroyed"):  # noqa
            next(it)


class AnyVectorTests(unittest.TestCase):
    def test_main(self):
        v = opentimelineio.core._core_utils.AnyVector()

        with self.assertRaises(IndexError):
            del v[0]  # There is a special case in the C++ code for empty vector

        v.append(1)
        self.assertEqual(len(v), 1)
        v.append(2)
        self.assertEqual(len(v), 2)

        self.assertEqual([value for value in v], [1, 2])

        v.insert(0, 5)
        self.assertEqual([value for value in v], [5, 1, 2])
        self.assertEqual(v[0], 5)
        self.assertEqual(v[-3], 5)

        with self.assertRaises(IndexError):
            v[100]

        with self.assertRaises(IndexError):
            v[-100]

        v[-1] = 100
        self.assertEqual(v[2], 100)

        with self.assertRaises(IndexError):
            v[-4] = -1

        with self.assertRaises(IndexError):
            v[100] = 100

        del v[0]
        self.assertEqual(len(v), 2)
        # Doesn't work...
        # assert v == [1, 100]
        self.assertEqual([value for value in v], [1, 100])

        del v[1000]  # This will surprisingly delete the last item...
        self.assertEqual(len(v), 1)
        self.assertEqual([value for value in v], [1])

        # Will delete the last item even if the index doesn't match.
        # It's a surprising behavior.
        # This is caused by size_t(index)
        del v[-1000]

        v.extend([1, '234', {}])

        items = []
        for value in iter(v):  # Test AnyVector.Iterator.iter
            items.append(value)

        self.assertEqual(items, [1, '234', {}])
        self.assertFalse(v == [1, '234', {}])  # __eq__ is not implemented

        self.assertTrue(1 in v)  # Test __contains__
        self.assertTrue('234' in v)
        self.assertTrue({} in v)
        self.assertFalse(5 in v)

        self.assertEqual(list(reversed(v)), [{}, '234', 1])

        self.assertEqual(v.index('234'), 1)

        v += [1, 2]
        self.assertEqual(v.count(1), 2)

        self.assertEqual(v + ['new'], [1, '234', {}, 1, 2, 'new'])  # __add__
        self.assertEqual(['new'] + v, [1, '234', {}, 1, 2, 'new'])  # __radd__

        self.assertEqual(v + ('new',), [1, '234', {}, 1, 2, 'new'])  # noqa __add__ with non list type

        v2 = opentimelineio.core._core_utils.AnyVector()
        v2.append('v2')

        self.assertEqual(v + v2, [1, '234', {}, 1, 2, 'v2'])  # __add__ with AnyVector

        with self.assertRaises(TypeError):
            v + 'asd'  # __add__ invalid type

        self.assertEqual(str(v), "[1, '234', {}, 1, 2]")
        self.assertEqual(repr(v), "[1, '234', {}, 1, 2]")

        v3 = opentimelineio.core._core_utils.AnyVector()
        v3.extend(range(10))
        self.assertEqual(v3[2:], [2, 3, 4, 5, 6, 7, 8, 9])
        self.assertEqual(v3[4:8], [4, 5, 6, 7])
        self.assertEqual(v3[1:7:2], [1, 3, 5])

        del v3[2:7]
        self.assertEqual(list(v3), [0, 1, 7, 8, 9])

        v4 = opentimelineio.core._core_utils.AnyVector()
        v4.extend(range(10))

        del v4[::2]
        self.assertEqual(list(v4), [1, 3, 5, 7, 9])

        v5 = opentimelineio.core._core_utils.AnyVector()
        tmplist = [1, 2]
        v5.append(tmplist)
        # If AnyVector was a pure list, this would fail. But it's not a real list.
        # Appending copies data, completely removing references to it.
        self.assertIsNot(v5[0], tmplist)

    def test_raises_if_ref_destroyed(self):
        v1 = opentimelineio.core._core_utils.AnyVector()
        opentimelineio._otio._testing.test_AnyVector_destroy(v1)

        with self.assertRaisesRegex(ValueError, r"Underlying C\+\+ AnyVector object has been destroyed"):  # noqa
            v1[0]

        v2 = opentimelineio.core._core_utils.AnyVector()
        opentimelineio._otio._testing.test_AnyVector_destroy(v2)

        with self.assertRaisesRegex(ValueError, r"Underlying C\+\+ AnyVector object has been destroyed"):  # noqa
            v2[0] = 1

        v3 = opentimelineio.core._core_utils.AnyVector()
        opentimelineio._otio._testing.test_AnyVector_destroy(v3)

        with self.assertRaisesRegex(ValueError, r"Underlying C\+\+ AnyVector object has been destroyed"):  # noqa
            del v3[0]

        v4 = opentimelineio.core._core_utils.AnyVector()
        v4.append(1)
        it = iter(v4)
        opentimelineio._otio._testing.test_AnyVector_destroy(v4)
        with self.assertRaisesRegex(ValueError, r"Underlying C\+\+ AnyVector object has been destroyed"):  # noqa
            next(it)

    def test_copy(self):
        list1 = [1, 2, [3, 4], 5]
        copied = copy.copy(list1)
        self.assertEqual(list(list1), list(copied))

        v = opentimelineio.core._core_utils.AnyVector()
        v.extend([1, 2, [3, 4], 5])

        copied = copy.copy(v)
        self.assertIsNot(v, copied)
        # AnyVector can only deep copy. So it's __copy__
        # does a deepcopy.
        self.assertIsNot(v[2], copied[2])

        deepcopied = copy.deepcopy(v)
        self.assertIsNot(v, deepcopied)
        self.assertIsNot(v[2], deepcopied[2])



#!/usr/bin/env python
#
# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""Tests for the OTIOD adapter."""

import unittest
import os
import tempfile

import opentimelineio as otio
from opentimelineio import test_utils as otio_test_utils
from opentimelineio.adapters import file_bundle_utils

SAMPLE_DATA_DIR = os.path.join(os.path.dirname(__file__), "sample_data")
SCREENING_EXAMPLE_PATH = os.path.join(SAMPLE_DATA_DIR, "screening_example.otio")

MEDIA_EXAMPLE_PATH_REL = os.path.relpath(
    os.path.join(
        SAMPLE_DATA_DIR,
        "OpenTimelineIO@3xDark.png"
    )
)
MEDIA_EXAMPLE_PATH_URL_REL = otio.url_utils.url_from_filepath(
    MEDIA_EXAMPLE_PATH_REL
)
MEDIA_EXAMPLE_PATH_ABS = os.path.abspath(
    MEDIA_EXAMPLE_PATH_REL.replace(
        "3xDark",
        "3xLight"
    )
)
MEDIA_EXAMPLE_PATH_URL_ABS = otio.url_utils.url_from_filepath(
    MEDIA_EXAMPLE_PATH_ABS
)


class OTIODTester(unittest.TestCase, otio_test_utils.OTIOAssertions):
    def setUp(self):
        tl = otio.adapters.read_from_file(SCREENING_EXAMPLE_PATH)

        # convert to contrived local reference
        last_rel = False
        for cl in tl.find_clips():
            # vary the relative and absolute paths, make sure that both work
            next_rel = (
                MEDIA_EXAMPLE_PATH_URL_REL if last_rel else MEDIA_EXAMPLE_PATH_URL_ABS
            )
            last_rel = not last_rel
            cl.media_reference = otio.schema.ExternalReference(
                target_url=next_rel
            )

        self.tl = tl

    def test_file_bundle_manifest_missing_reference(self):
        # all missing should be empty
        result_otio, manifest = (
            file_bundle_utils._prepped_otio_for_bundle_and_manifest(
                input_otio=self.tl,
                media_policy=file_bundle_utils.MediaReferencePolicy.AllMissing,
                adapter_name="TEST_NAME",
            )
        )

        self.assertEqual(manifest, {})
        for cl in result_otio.find_clips():
            self.assertIsInstance(
                cl.media_reference,
                otio.schema.MissingReference,
                "{} is of type {}, not an instance of {}.".format(
                    cl.media_reference,
                    type(cl.media_reference),
                    type(otio.schema.MissingReference)
                )
            )

    def test_file_bundle_manifest(self):
        result_otio, manifest = (
            file_bundle_utils._prepped_otio_for_bundle_and_manifest(
                input_otio=self.tl,
                media_policy=(
                    file_bundle_utils.MediaReferencePolicy.ErrorIfNotFile
                ),
                adapter_name="TEST_NAME",
            )
        )

        self.assertEqual(len(manifest.keys()), 2)

        files_in_manifest = set(manifest.keys())
        known_files = {
            MEDIA_EXAMPLE_PATH_ABS: 5,
            os.path.abspath(MEDIA_EXAMPLE_PATH_REL): 4
        }

        # should only contain absolute paths
        self.assertEqual(files_in_manifest, set(known_files.keys()))

        for fname, count in known_files.items():
            self.assertEqual(len(manifest[fname]), count)

    def test_round_trip(self):
        with tempfile.NamedTemporaryFile(suffix=".otiod") as bogusfile:
            tmp_path = bogusfile.name
        otio.adapters.write_to_file(self.tl, tmp_path)
        self.assertTrue(os.path.exists(tmp_path))

        # by default will provide relative paths
        result = otio.adapters.read_from_file(
            tmp_path,
        )

        for cl in result.find_clips():
            self.assertNotEqual(
                cl.media_reference.target_url,
                MEDIA_EXAMPLE_PATH_URL_REL
            )

        # conform media references in input to what they should be in the output
        for cl in self.tl.find_clips():
            # construct an absolute file path to the result
            cl.media_reference.target_url = (
                otio.url_utils.url_from_filepath(
                    os.path.join(
                        otio.adapters.file_bundle_utils.BUNDLE_DIR_NAME,
                        os.path.basename(cl.media_reference.target_url)
                    )
                )
            )

        self.assertJsonEqual(result, self.tl)

    def test_round_trip_all_missing_references(self):
        with tempfile.NamedTemporaryFile(suffix=".otiod") as bogusfile:
            tmp_path = bogusfile.name
        otio.adapters.write_to_file(
            self.tl,
            tmp_path,
            media_policy=(
                otio.adapters.file_bundle_utils.MediaReferencePolicy.AllMissing
            )
        )

        # ...but can be optionally told to generate absolute paths
        result = otio.adapters.read_from_file(
            tmp_path,
            absolute_media_reference_paths=True
        )

        for cl in result.find_clips():
            self.assertIsInstance(
                cl.media_reference,
                otio.schema.MissingReference
            )

    def test_round_trip_absolute_paths(self):
        with tempfile.NamedTemporaryFile(suffix=".otiod") as bogusfile:
            tmp_path = bogusfile.name
        otio.adapters.write_to_file(self.tl, tmp_path)

        # ...but can be optionally told to generate absolute paths
        result = otio.adapters.read_from_file(
            tmp_path,
            absolute_media_reference_paths=True
        )

        for cl in result.find_clips():
            self.assertNotEqual(
                cl.media_reference.target_url,
                MEDIA_EXAMPLE_PATH_URL_REL
            )

        # conform media references in input to what they should be in the output
        for cl in self.tl.find_clips():
            # should be only field that changed
            cl.media_reference.target_url = (
                otio.url_utils.url_from_filepath(
                    os.path.join(
                        tmp_path,
                        otio.adapters.file_bundle_utils.BUNDLE_DIR_NAME,
                        os.path.basename(cl.media_reference.target_url)
                    )
                )
            )

        self.assertJsonEqual(result, self.tl)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
#
# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import unittest
import os
from pathlib import Path
import sys

from unittest import mock

from importlib import reload as import_reload

try:
    import importlib.metadata as metadata
except ImportError:
    # For python 3.7
    import importlib_metadata as metadata

import opentimelineio as otio
from tests import baseline_reader


class TestSetuptoolsPlugin(unittest.TestCase):
    def setUp(self):
        # Get the location of the mock plugin module metadata
        mock_module_path = os.path.join(
            os.path.normpath(baseline_reader.path_to_baseline_directory()),
            'plugin_module',
        )
        self.mock_module_manifest_path = Path(
            mock_module_path,
            "otio_jsonplugin",
            "plugin_manifest.json"
        ).absolute().as_posix()

        self.override_adapter_manifest_path = Path(
            mock_module_path,
            "otio_override_adapter",
            "plugin_manifest.json"
        ).absolute().as_posix()

        # Create a WorkingSet as if the module were installed
        entries = [mock_module_path] + sys.path

        self.original_sysmodule_keys = set(sys.modules.keys())

        self.sys_patch = mock.patch('sys.path', entries)
        self.sys_patch.start()

    def tearDown(self):
        self.sys_patch.stop()

        # Remove any modules added under test.  We cannot replace sys.modules with
        # a copy from setUp. For more, see: https://bugs.python.org/msg188914
        for key in set(sys.modules.keys()) ^ self.original_sysmodule_keys:
            sys.modules.pop(key)

    def test_detect_plugin(self):
        """This manifest uses the plugin_manifest function"""

        # Create a manifest and ensure it detected the mock adapter and linker
        man = otio.plugins.manifest.load_manifest()

        # Make sure the adapter is included in the adapter list
        adapter_names = [adapter.name for adapter in man.adapters]
        self.assertIn('mock_adapter', adapter_names)

        # Make sure the linker is included in the linker list
        linker_names = [linker.name for linker in man.media_linkers]
        self.assertIn('mock_linker', linker_names)

        # Make sure adapters and linkers landed in the proper place
        for adapter in man.adapters:
            self.assertIsInstance(adapter, otio.adapters.Adapter)

        for linker in man.media_linkers:
            self.assertIsInstance(linker, otio.media_linker.MediaLinker)

    def test_override_adapter(self):

        # Test that entrypoint plugins load before builtin
        man = otio.plugins.manifest.load_manifest()

        # The override_adapter creates another otiod adapter
        adapters = [adapter for adapter in man.adapters
                    if adapter.name == "otiod"]

        # More then one otiod adapter should exist.
        self.assertTrue(len(adapters) > 1)

        # Override adapter should be the first adapter found
        manifest = adapters[0].plugin_info_map().get('from manifest', None)
        self.assertEqual(manifest, self.override_adapter_manifest_path)

        self.assertTrue(
            any(
                True for p in man.source_files
                if self.override_adapter_manifest_path in p
            )
        )

    def test_entrypoints_disabled(self):
        os.environ["OTIO_DISABLE_ENTRYPOINTS_PLUGINS"] = "1"
        import_reload(otio.plugins.manifest)

        # detection of the environment variable happens on import, force a
        # reload to ensure that it is triggered
        with self.assertRaises(AssertionError):
            self.test_detect_plugin()

        # override adapter should not be loaded either
        with self.assertRaises(AssertionError):
            self.test_override_adapter()

        # remove the environment variable and reload again for usage in the
        # other tests
        del os.environ["OTIO_DISABLE_ENTRYPOINTS_PLUGINS"]
        import_reload(otio.plugins.manifest)

    def test_detect_plugin_json_manifest(self):
        # Test detecting a plugin that rather than exposing the plugin_manifest
        # function, just simply has a plugin_manifest.json provided at the
        # package top level.
        man = otio.plugins.manifest.load_manifest()

        # Make sure the adapter is included in the adapter list
        adapter_names = [adapter.name for adapter in man.adapters]
        self.assertIn('mock_adapter_json', adapter_names)

        # Make sure the linker is included in the linker list
        linker_names = [linker.name for linker in man.media_linkers]
        self.assertIn('mock_linker_json', linker_names)

        # Make sure adapters and linkers landed in the proper place
        for adapter in man.adapters:
            self.assertIsInstance(adapter, otio.adapters.Adapter)

        for linker in man.media_linkers:
            self.assertIsInstance(linker, otio.media_linker.MediaLinker)

        self.assertTrue(
            any(
                True for p in man.source_files
                if self.mock_module_manifest_path in p
            )
        )

    def test_deduplicate_env_variable_paths(self):
        "Ensure that duplicate entries in the environment variable are ignored"

        # back up existing manifest
        bak_env = os.environ.get('OTIO_PLUGIN_MANIFEST_PATH')

        relative_path = self.mock_module_manifest_path.replace(os.getcwd(), '.')

        # set where to find the new manifest
        os.environ['OTIO_PLUGIN_MANIFEST_PATH'] = os.pathsep.join(
            (
                # absolute
                self.mock_module_manifest_path,

                # relative
                relative_path
            )
        )

        result = otio.plugins.manifest.load_manifest()
        self.assertEqual(
            len(
                [
                    p for p in result.source_files
                    if self.mock_module_manifest_path in p
                ]
            ),
            1
        )
        if relative_path != self.mock_module_manifest_path:
            self.assertNotIn(relative_path, result.source_files)

        if bak_env:
            os.environ['OTIO_PLUGIN_MANIFEST_PATH'] = bak_env
        else:
            del os.environ['OTIO_PLUGIN_MANIFEST_PATH']

    def test_plugin_load_failure(self):
        """When a plugin fails to load, ensure the exception message
        is logged (and no exception thrown)
        """

        sys.modules['otio_mock_bad_module'] = mock.Mock(
            name='otio_mock_bad_module',
            plugin_manifest=mock.Mock(
                side_effect=Exception("Mock Exception")
            )
        )

        entry_points = mock.patch(
            'opentimelineio.plugins.manifest.metadata.entry_points',
            return_value=[
                metadata.EntryPoint(
                    'mock_bad_module',
                    'otio_mock_bad_module',
                    'opentimelineio.plugins'
                )
            ]
        )

        with self.assertLogs() as cm, entry_points:
            # Load the above mock entrypoint, expect it to fail and log
            otio.plugins.manifest.load_manifest()

            load_errors = [
                r for r in cm.records
                if r.message.startswith(
                    "could not load plugin: mock_bad_module.  "
                    "Exception is: Mock Exception"
                )
            ]
            self.assertEqual(len(load_errors), 1)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""Tests for the OTIOZ adapter."""

import unittest
import os
import tempfile
import shutil

import urllib.parse as urlparse

import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils

SAMPLE_DATA_DIR = os.path.join(os.path.dirname(__file__), "sample_data")
SCREENING_EXAMPLE_PATH = os.path.join(SAMPLE_DATA_DIR, "screening_example.otio")
MEDIA_EXAMPLE_PATH_REL = os.path.relpath(
    os.path.join(
        SAMPLE_DATA_DIR,
        "OpenTimelineIO@3xDark.png"
    )
)
MEDIA_EXAMPLE_PATH_URL_REL = otio.url_utils.url_from_filepath(
    MEDIA_EXAMPLE_PATH_REL
)
MEDIA_EXAMPLE_PATH_ABS = os.path.abspath(
    MEDIA_EXAMPLE_PATH_REL.replace(
        "3xDark",
        "3xLight"
    )
)
MEDIA_EXAMPLE_PATH_URL_ABS = otio.url_utils.url_from_filepath(
    MEDIA_EXAMPLE_PATH_ABS
)


class OTIOZTester(unittest.TestCase, otio_test_utils.OTIOAssertions):
    def setUp(self):
        tl = otio.adapters.read_from_file(SCREENING_EXAMPLE_PATH)

        # convert to contrived local reference
        last_rel = False
        for cl in tl.find_clips():
            # vary the relative and absolute paths, make sure that both work
            next_rel = (
                MEDIA_EXAMPLE_PATH_URL_REL
                if last_rel else MEDIA_EXAMPLE_PATH_URL_ABS
            )
            last_rel = not last_rel
            cl.media_reference = otio.schema.ExternalReference(
                target_url=next_rel
            )

        self.tl = tl

    def test_dryrun(self):
        # generate a fake name
        with tempfile.NamedTemporaryFile(suffix=".otioz") as bogusfile:
            fname = bogusfile.name

        # dryrun should compute what the total size of the zipfile will be.
        size = otio.adapters.write_to_file(self.tl, fname, dryrun=True)
        self.assertEqual(
            size,
            os.path.getsize(MEDIA_EXAMPLE_PATH_ABS) +
            os.path.getsize(MEDIA_EXAMPLE_PATH_REL)
        )

    def test_not_a_file_error(self):
        # dryrun should compute what the total size of the zipfile will be.
        tmp_path = tempfile.mkstemp(suffix=".otioz", text=False)[1]
        with tempfile.NamedTemporaryFile() as bogusfile:
            fname = bogusfile.name
        for cl in self.tl.find_clips():
            # write with a non-file schema
            cl.media_reference = otio.schema.ExternalReference(
                target_url=f"http://{fname}"
            )
        with self.assertRaises(otio.exceptions.OTIOError):
            otio.adapters.write_to_file(self.tl, tmp_path, dryrun=True)

        for cl in self.tl.find_clips():
            cl.media_reference = otio.schema.ExternalReference(
                target_url=otio.url_utils.url_from_filepath(fname)
            )
        with self.assertRaises(otio.exceptions.OTIOError):
            otio.adapters.write_to_file(self.tl, tmp_path, dryrun=True)

        tempdir = tempfile.mkdtemp()
        fname = tempdir
        shutil.rmtree(tempdir)
        for cl in self.tl.find_clips():
            cl.media_reference = otio.schema.ExternalReference(target_url=fname)

    def test_colliding_basename(self):
        tempdir = tempfile.mkdtemp()
        new_path = os.path.join(
            tempdir,
            os.path.basename(MEDIA_EXAMPLE_PATH_ABS)
        )
        shutil.copyfile(
            MEDIA_EXAMPLE_PATH_ABS,
            new_path
        )
        list(self.tl.find_clips())[0].media_reference.target_url = (
            otio.url_utils.url_from_filepath(new_path)
        )

        tmp_path = tempfile.mkstemp(suffix=".otioz", text=False)[1]
        with self.assertRaises(otio.exceptions.OTIOError):
            otio.adapters.write_to_file(self.tl, tmp_path)

        with self.assertRaises(otio.exceptions.OTIOError):
            otio.adapters.write_to_file(self.tl, tmp_path, dryrun=True)

        shutil.rmtree(tempdir)

    def test_round_trip(self):
        with tempfile.NamedTemporaryFile(suffix=".otioz") as bogusfile:
            tmp_path = bogusfile.name
        otio.adapters.write_to_file(self.tl, tmp_path)
        self.assertTrue(os.path.exists(tmp_path))

        result = otio.adapters.read_from_file(tmp_path)

        for cl in result.find_clips():
            self.assertNotIn(
                cl.media_reference.target_url,
                [MEDIA_EXAMPLE_PATH_URL_ABS, MEDIA_EXAMPLE_PATH_URL_REL]
            )
            # ensure that unix style paths are used, so that bundles created on
            # windows are compatible with ones created on unix
            self.assertFalse(
                urlparse.urlparse(
                    cl.media_reference.target_url
                ).path.startswith(
                    "media\\"
                )
            )

        # conform media references in input to what they should be in the output
        for cl in self.tl.find_clips():
            # should be only field that changed
            cl.media_reference.target_url = "media/{}".format(
                os.path.basename(cl.media_reference.target_url)
            )

        self.assertJsonEqual(result, self.tl)

    def test_round_trip_with_extraction(self):
        with tempfile.NamedTemporaryFile(suffix=".otioz") as bogusfile:
            tmp_path = bogusfile.name
        otio.adapters.write_to_file(self.tl, tmp_path)
        self.assertTrue(os.path.exists(tmp_path))

        tempdir = tempfile.mkdtemp()
        result = otio.adapters.read_from_file(
            tmp_path,
            extract_to_directory=tempdir
        )

        # make sure that all the references are ExternalReference
        for cl in result.find_clips():
            self.assertIsInstance(
                cl.media_reference,
                otio.schema.ExternalReference
            )

        # conform media references in input to what they should be in the output
        for cl in self.tl.find_clips():
            # should be only field that changed
            cl.media_reference.target_url = "media/{}".format(
                os.path.basename(cl.media_reference.target_url)
            )

        self.assertJsonEqual(result, self.tl)

        # content file
        self.assertTrue(
            os.path.exists(
                os.path.join(
                    tempdir,
                    otio.adapters.file_bundle_utils.BUNDLE_PLAYLIST_PATH
                )
            )
        )

        # media directory overall
        self.assertTrue(
            os.path.exists(
                os.path.join(
                    tempdir,
                    otio.adapters.file_bundle_utils.BUNDLE_DIR_NAME
                )
            )
        )

        # actual media file
        self.assertTrue(
            os.path.exists(
                os.path.join(
                    tempdir,
                    otio.adapters.file_bundle_utils.BUNDLE_DIR_NAME,
                    os.path.basename(MEDIA_EXAMPLE_PATH_URL_REL)
                )
            )
        )

    def test_round_trip_with_extraction_no_media(self):
        with tempfile.NamedTemporaryFile(suffix=".otioz") as bogusfile:
            tmp_path = bogusfile.name
        otio.adapters.write_to_file(
            self.tl,
            tmp_path,
            media_policy=(
                otio.adapters.file_bundle_utils.MediaReferencePolicy.AllMissing
            ),
        )

        tempdir = tempfile.mkdtemp()
        result = otio.adapters.read_from_file(
            tmp_path,
            extract_to_directory=tempdir,
        )

        version_file_path = os.path.join(
            tempdir,
            otio.adapters.file_bundle_utils.BUNDLE_VERSION_FILE
        )
        self.assertTrue(os.path.exists(version_file_path))
        with open(version_file_path) as fi:
            self.assertEqual(
                fi.read(),
                otio.adapters.file_bundle_utils.BUNDLE_VERSION
            )

        # conform media references in input to what they should be in the output
        for cl in result.find_clips():
            # should be all MissingReferences
            self.assertIsInstance(
                cl.media_reference,
                otio.schema.MissingReference
            )
            self.assertIn("original_target_url", cl.media_reference.metadata)


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import unittest

import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils


class EffectTest(unittest.TestCase, otio_test_utils.OTIOAssertions):

    def test_cons(self):
        ef = otio.schema.Effect(
            name="blur it",
            effect_name="blur",
            metadata={"foo": "bar"}
        )
        encoded = otio.adapters.otio_json.write_to_string(ef)
        decoded = otio.adapters.otio_json.read_from_string(encoded)
        self.assertIsOTIOEquivalentTo(ef, decoded)
        self.assertEqual(decoded.name, "blur it")
        self.assertEqual(decoded.effect_name, "blur")
        self.assertEqual(decoded.metadata['foo'], 'bar')

    def test_eq(self):
        ef = otio.schema.Effect(
            name="blur it",
            effect_name="blur",
            metadata={"foo": "bar"}
        )
        ef2 = otio.schema.Effect(
            name="blur it",
            effect_name="blur",
            metadata={"foo": "bar"}
        )
        self.assertIsOTIOEquivalentTo(ef, ef2)

    def test_str(self):
        ef = otio.schema.Effect(
            name="blur it",
            effect_name="blur",
            metadata={"foo": "bar"}
        )
        self.assertMultiLineEqual(
            str(ef),
            "Effect({}, {}, {})".format(
                str(ef.name),
                str(ef.effect_name),
                str(ef.metadata)
            )
        )
        self.assertMultiLineEqual(
            repr(ef),
            "otio.schema.Effect("
            "name={}, "
            "effect_name={}, "
            "metadata={}"
            ")".format(
                repr(ef.name),
                repr(ef.effect_name),
                repr(ef.metadata),
            )
        )

    def test_setters(self):
        ef = otio.schema.Effect(
            name="blur it",
            effect_name="blur",
            metadata={"foo": "bar"}
        )
        self.assertEqual(ef.effect_name, "blur")
        ef.effect_name = "flop"
        self.assertEqual(ef.effect_name, "flop")


class TestLinearTimeWarp(unittest.TestCase, otio_test_utils.OTIOAssertions):
    def test_cons(self):
        ef = otio.schema.LinearTimeWarp("Foo", 2.5, {'foo': 'bar'})
        self.assertEqual(ef.effect_name, "LinearTimeWarp")
        self.assertEqual(ef.name, "Foo")
        self.assertEqual(ef.time_scalar, 2.5)
        self.assertEqual(ef.metadata, {"foo": "bar"})

    def test_serialize(self):
        ef = otio.schema.LinearTimeWarp("Foo", 2.5, {'foo': 'bar'})
        encoded = otio.adapters.otio_json.write_to_string(ef)
        decoded = otio.adapters.otio_json.read_from_string(encoded)
        self.assertIsOTIOEquivalentTo(ef, decoded)

    def test_setters(self):
        ef = otio.schema.LinearTimeWarp("Foo", 2.5, {'foo': 'bar'})
        self.assertEqual(ef.time_scalar, 2.5)
        ef.time_scalar = 5.0
        self.assertEqual(ef.time_scalar, 5.0)


class TestFreezeFrame(unittest.TestCase):
    def test_cons(self):
        ef = otio.schema.FreezeFrame("Foo", {'foo': 'bar'})
        self.assertEqual(ef.effect_name, "FreezeFrame")
        self.assertEqual(ef.name, "Foo")
        self.assertEqual(ef.time_scalar, 0)
        self.assertEqual(ef.metadata, {"foo": "bar"})


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import sys
import shutil
import tempfile
import unittest

import opentimelineio as otio


class TestCoreFunctions(unittest.TestCase):
    def setUp(self):
        self.tmpDir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tmpDir)

    def test_deserialize_json_from_file_errors(self):
        """Verify the bindings return the correct errors based on the errno"""

        with self.assertRaises(FileNotFoundError) as exc:
            otio.core.deserialize_json_from_file('non-existent-file-here')
        self.assertIsInstance(exc.exception, FileNotFoundError)

    @unittest.skipUnless(
        not sys.platform.startswith("win"),
        "requires non Windows system"
    )
    def test_serialize_json_to_file_errors_non_windows(self):
        """Verify the bindings return the correct errors based on the errno"""

        with self.assertRaises(IsADirectoryError) as exc:
            otio.core.serialize_json_to_file({}, self.tmpDir)
        self.assertIsInstance(exc.exception, IsADirectoryError)

    @unittest.skipUnless(sys.platform.startswith("win"), "requires Windows")
    def test_serialize_json_to_file_errors_windows(self):
        """Verify the bindings return the correct errors based on the errno"""

        with self.assertRaises(PermissionError) as exc:
            otio.core.serialize_json_to_file({}, self.tmpDir)
        self.assertIsInstance(exc.exception, PermissionError)



#!/usr/bin/env python
#
# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import opentimelineio as otio
import sys
import copy

inputpath, outputpath = sys.argv[1:]

# Read the file
timeline = otio.adapters.read_from_file(inputpath)
video_tracks = timeline.video_tracks()
audio_tracks = timeline.audio_tracks()

print("Read {} video tracks and {} audio tracks.".format(
    len(video_tracks),
    len(audio_tracks)
))

# Take just the video tracks - and flatten them into one.
# This will trim away any overlapping segments, collapsing everything
# into a single track.
print(f"Flattening {len(video_tracks)} video tracks into one...")
onetrack = otio.algorithms.flatten_stack(video_tracks)

# Now make a new empty Timeline and put that one Track into it
newtimeline = otio.schema.Timeline(name=f"{timeline.name} Flattened")
newtimeline.tracks[:] = [onetrack]

# keep the audio track(s) as-is
newtimeline.tracks.extend(copy.deepcopy(audio_tracks))

# ...and save it to disk.
print("Saving {} video tracks and {} audio tracks.".format(
    len(newtimeline.video_tracks()),
    len(newtimeline.audio_tracks())
))
otio.adapters.write_to_file(newtimeline, outputpath)



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project
import opentimelineio as otio

"""
This is the implementation of the contrived example adapter that simply writes
the number of tracks in the timeline to a file, or will read an integer from a
file and create a timeline with that number of tracks.

This would be where your plugin implementation would be.
"""


def write_to_string(input_otio):
    return f'{len(input_otio.tracks)}'


def read_from_string(input_str):
    t = otio.schema.Timeline()

    for i in range(int(input_str)):
        t.tracks.append(otio.schema.Track())

    return t



#! /usr/bin/env python
#
# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""Setup.py for installing OpenTimelineIO

For more information:
- see README.md
- http://opentimeline.io
"""

import multiprocessing
import os
import shlex
import sys
import platform
import subprocess
import tempfile
import shutil

from setuptools import (
    setup,
    Extension,
    find_packages,
)

import setuptools.command.build_ext
import setuptools.command.build_py

SOURCE_DIR = os.path.abspath(os.path.dirname(__file__))

PLAT_TO_CMAKE = {
    "win32": "Win32",
    "win-amd64": "x64",
}


def _debugInstance(x):
    for a in sorted(dir(x)):
        print("{}:     {}".format(a, getattr(x, a)))


def join_args(args):
    return ' '.join(map(shlex.quote, args))


class OTIO_build_ext(setuptools.command.build_ext.build_ext):
    """
    def initialize_options(self):
        super(setuptools.command.build_ext.build_ext, self).initialize_options()
    """

    built = False

    def run(self):
        self.announce('running OTIO build_ext', level=2)
        # Let the original build_ext class do its job.
        # This is rather important because build_ext.run takes care of a
        # couple of things, one of which is to copy the built files into
        # the source tree (in src/py-opentimelineio/opentimelineio)
        # when building in editable mode.
        super().run()

    def build_extension(self, _ext: Extension):
        # This works around the fact that we build _opentime and _otio
        # extensions as a one-shot cmake invocation. Setuptools calls
        # build_extension for each Extension registered in the setup function.
        if not self.built:
            self.build()
            self.built = True

    def build(self):
        self.build_temp_dir = (
            os.environ.get("OTIO_CXX_BUILD_TMP_DIR")
            or os.path.abspath(self.build_temp)
        )

        if not os.path.exists(self.build_temp_dir):
            os.makedirs(self.build_temp_dir)

        debug = (self.debug or bool(os.environ.get("OTIO_CXX_DEBUG_BUILD")))
        self.build_config = ('Debug' if debug else 'Release')

        self.cmake_preflight_check()
        self.cmake_generate()
        self.cmake_install()

    def is_windows(self):
        return platform.system() == "Windows"

    def is_mingw(self):
        return self.plat_name.startswith('mingw')

    def generate_cmake_arguments(self):
        # Use the provided build dir so setuptools will be able to locate and
        # either install to the correct location or package.
        install_dir = os.path.abspath(self.build_lib)
        if not install_dir.endswith(os.path.sep):
            install_dir += os.path.sep

        cmake_args = [
            # Python_EXECUTABLE is important as it tells CMake's FindPython
            # which Python executable to use. We absolutely want to use the
            # interpreter that was used to execute the setup.py.
            # See https://cmake.org/cmake/help/v3.20/module/FindPython.html#artifacts-specification # noqa: E501
            # Also, be careful, CMake is case sensitive ;)
            '-DPython_EXECUTABLE=' + sys.executable,
            '-DOTIO_PYTHON_INSTALL:BOOL=ON',
            '-DOTIO_CXX_INSTALL:BOOL=OFF',
            '-DOTIO_SHARED_LIBS:BOOL=OFF',
            '-DCMAKE_BUILD_TYPE=' + self.build_config,
            '-DOTIO_PYTHON_INSTALL_DIR=' + install_dir,
            # turn off the C++ tests during a Python build
            '-DBUILD_TESTING:BOOL=OFF',
            # Python modules wil be installed by setuptools.
            '-DOTIO_INSTALL_PYTHON_MODULES:BOOL=OFF',
        ]
        if self.is_windows():
            if self.is_mingw():
                cmake_args += ['-G Unix Makefiles']
            else:
                cmake_args += ["-A", PLAT_TO_CMAKE[self.plat_name]]

        cxx_coverage = bool(os.environ.get("OTIO_CXX_COVERAGE_BUILD"))
        if cxx_coverage and not os.environ.get("OTIO_CXX_BUILD_TMP_DIR"):
            raise RuntimeError(
                "C++ code coverage requires that both OTIO_CXX_COVERAGE_BUILD=ON "
                "and OTIO_CXX_BUILD_TMP_DIR are specified as environment "
                "variables, otherwise coverage cannot be generated."
            )

        if cxx_coverage:
            cmake_args += ['-DOTIO_CXX_COVERAGE=1']

        # allow external arguments to cmake via the CMAKE_ARGS env var
        cmake_args += [
            arg for arg in os.environ.get("CMAKE_ARGS", "").split(" ")
            if arg
        ]

        return cmake_args

    def cmake_preflight_check(self):
        """
        Verify that CMake is greater or equal to the required version
        We do this so that the error message is clear if the minimum version is not met.
        """
        self.announce('running cmake check', level=2)
        # We need to run cmake --check-system-vars because it will still generate
        # a CMakeCache.txt file.
        tmpdir = tempfile.mkdtemp(dir=self.build_temp_dir)

        args = ["--check-system-vars", SOURCE_DIR] + self.generate_cmake_arguments()

        proc = subprocess.Popen(
            ["cmake"] + args,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            cwd=tmpdir,
            universal_newlines=True
        )

        _, stderr = proc.communicate()
        if proc.returncode != 0:
            raise RuntimeError(stderr.strip())

        shutil.rmtree(tmpdir)

    def cmake_generate(self):
        self.announce('running cmake generation', level=2)

        cmake_args = ['cmake', SOURCE_DIR] + self.generate_cmake_arguments()
        self.announce(join_args(cmake_args), level=2)

        subprocess.check_call(
            cmake_args,
            cwd=self.build_temp_dir,
            env=os.environ.copy()
        )

    def cmake_install(self):
        self.announce('running cmake build', level=2)
        if self.is_windows() and not self.is_mingw():
            multi_proc = '/m'
        else:
            multi_proc = f'-j{multiprocessing.cpu_count()}'

        cmake_args = [
            'cmake',
            '--build', '.',
            '--target', 'install',
            '--config', self.build_config,
            '--', multi_proc,
        ]

        self.announce(join_args(cmake_args), level=2)
        subprocess.check_call(
            cmake_args,
            cwd=self.build_temp_dir,
            env=os.environ.copy()
        )


# check the python version first
if (
    sys.version_info[0] < 3 or
    (sys.version_info[0] == 3 and sys.version_info[1] < 7)
):
    sys.exit(
        'OpenTimelineIO requires python3.7 or greater, detected version:'
        ' {}.{}'.format(
            sys.version_info[0],
            sys.version_info[1]
        )
    )


# Metadata that gets stamped into the __init__ files during the build phase.
PROJECT_METADATA = {
    "version": "0.17.0",
    "author": 'Contributors to the OpenTimelineIO project',
    "author_email": 'otio-discussion@lists.aswf.io',
    "license": 'Apache 2.0 License',
}

METADATA_TEMPLATE = """
__version__ = "{version}"
__author__ = "{author}"
__author_email__ = "{author_email}"
__license__ = "{license}"
"""


def _append_version_info_to_init_scripts(build_lib):
    """Stamp PROJECT_METADATA into __init__ files."""

    for module, parentdir in [
            ("opentimelineio", "src/py-opentimelineio"),
            ("opentimelineview", "src")
    ]:
        target_file = os.path.join(build_lib, module, "__init__.py")
        source_file = os.path.join(
            os.path.dirname(__file__),
            parentdir,
            module,
            "__init__.py"
        )

        # get the base data from the original file
        with open(source_file) as fi:
            src_data = fi.read()

        # write that + the suffix to the target file
        with open(target_file, 'w') as fo:
            fo.write(src_data)
            fo.write(METADATA_TEMPLATE.format(**PROJECT_METADATA))


class OTIO_build_py(setuptools.command.build_py.build_py):
    """Stamps PROJECT_METADATA into __init__ files."""

    def run(self):
        super().run()

        # editable_mode isn't always present
        try:
            editable_mode = self.editable_mode
        except AttributeError:
            editable_mode = False

        if not self.dry_run and not editable_mode:
            # Only run when not in dry-mode (a dry run should not have any side effect)
            # and in non-editable mode. We don't want to edit files when in editable
            # mode because that could lead to modifications to the source files.
            # Note that setuptools will set self.editable_mode to True
            # when "pip install -e ." is run.
            _append_version_info_to_init_scripts(self.build_lib)


# copied from first paragraph of README.md
LONG_DESCRIPTION = """OpenTimelineIO is an interchange format and API for
editorial cut information. OTIO is not a container format for media, rather it
contains information about the order and length of cuts and references to
external media.

OTIO includes both a file format and an API for manipulating that format. It
also includes a plugin architecture for writing adapters to convert from/to
existing editorial timeline formats. It also implements a dependency- less
library for dealing strictly with time, opentime.

You can provide adapters for your video editing tool or pipeline as needed.
Each adapter allows for import/export between that proprietary tool and the
OpenTimelineIO format."""

setup(
    name='OpenTimelineIO',
    description='Editorial interchange format and API',
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    url='http://opentimeline.io',
    project_urls={
        'Source':
            'https://github.com/AcademySoftwareFoundation/OpenTimelineIO',
        'Documentation':
            'https://opentimelineio.readthedocs.io/',
        'Issues':
            'https://github.com/AcademySoftwareFoundation/OpenTimelineIO/issues',
    },

    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Multimedia :: Graphics',
        'Topic :: Multimedia :: Video',
        'Topic :: Multimedia :: Video :: Display',
        'Topic :: Multimedia :: Video :: Non-Linear Editor',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Operating System :: OS Independent',
        'Natural Language :: English',
    ],

    keywords='film tv editing editorial edit non-linear edl time',

    platforms='any',

    package_data={
        'opentimelineio': [
            'adapters/builtin_adapters.plugin_manifest.json',
        ],
    },

    packages=(
        find_packages(where="src/py-opentimelineio") +  # opentimelineio
        find_packages(where="src")  # opentimelineview
    ),

    ext_modules=[
        # The full and correct module name is required here because
        # setuptools needs to resolve the name to find the built file
        # and copy it into the source tree. (Because yes, editable install
        # means that the install should point to the source tree, which
        # means the .sos need to also be there alongside the python files).
        Extension('opentimelineio._otio', sources=[]),
        Extension('opentimelineio._opentime', sources=[]),
    ],

    package_dir={
        'opentimelineio': 'src/py-opentimelineio/opentimelineio',
        'opentimelineview': 'src/opentimelineview',
    },

    # Disallow 3.9.0 because of https://github.com/python/cpython/pull/22670
    python_requires='>=3.7, !=3.9.0',  # noqa: E501

    install_requires=[
        'importlib_metadata>=1.4; python_version < "3.8"',
    ],
    entry_points={
        'console_scripts': [
            'otiocat = opentimelineio.console.otiocat:main',
            'otioconvert = opentimelineio.console.otioconvert:main',
            'otiopluginfo = opentimelineio.console.otiopluginfo:main',
            'otiostat = opentimelineio.console.otiostat:main',
            'otiotool = opentimelineio.console.otiotool:main',
            'otioview = opentimelineview.console:main',
            (
                'otioautogen_serialized_schema_docs = '
                'opentimelineio.console.autogen_serialized_datamodel:main'
            ),
        ],
    },
    extras_require={
        'dev': [
            'check-manifest',
            'flake8>=3.5',
            'coverage>=4.5',
            'urllib3>=1.24.3'
        ],
        'view': [
            'PySide2~=5.11; platform.machine=="x86_64"',
            'PySide6~=6.2; platform.machine=="aarch64"'
        ]
    },

    # because we need to open() the adapters manifest, we aren't zip-safe
    zip_safe=False,

    # The sequence of operations performed by setup.py is:
    #   OTIO_install::initialize
    #   OTIO_install::run
    #   OTIO_build_py::run
    #   the OpenTimelineIO egg is created
    #   OTIO_build_ext::initialize_options
    #   MANIFEST.in is read
    #   the lack of CHANGELOG.md is then reported.
    #   OTIO_build_ext::run is called
    #   OTIO_build_ext::build
    #   site-packages/opentimelineio* is populated with all scripts and extensions.
    #   pyc's are created for every python script.
    #   The egg is moved into site-packages,
    #   wrapper scripts for all the otiotools are created in a bin directory at the
    #   installation root.

    cmdclass={
        'build_py': OTIO_build_py,
        'build_ext': OTIO_build_ext,
    },

    # expand the project metadata dictionary to fill in those values
    **PROJECT_METADATA
)



#!/usr/bin/env python
#
# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""Test file for the stack algorithms library."""

import unittest
import os
import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils


SAMPLE_DATA_DIR = os.path.join(os.path.dirname(__file__), "sample_data")
MULTITRACK_EXAMPLE_PATH = os.path.join(SAMPLE_DATA_DIR, "multitrack.otio")
PREFLATTENED_EXAMPLE_PATH = os.path.join(SAMPLE_DATA_DIR, "preflattened.otio")


class StackAlgoTests(unittest.TestCase, otio_test_utils.OTIOAssertions):
    """ test harness for stack algo functions """

    def setUp(self):
        self.trackZ = otio.adapters.read_from_string("""
        {
            "OTIO_SCHEMA": "Track.1",
            "children": [
                {
                    "OTIO_SCHEMA": "Clip.1",
                    "effects": [],
                    "markers": [],
                    "media_reference": null,
                    "metadata": {},
                    "name": "Z",
                    "source_range": {
                        "OTIO_SCHEMA": "TimeRange.1",
                        "duration": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 150
                        },
                        "start_time": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 0.0
                        }
                    }
                }
            ],
            "effects": [],
            "kind": "Video",
            "markers": [],
            "metadata": {},
            "name": "Sequence1",
            "source_range": null
        }
        """, "otio_json")

        self.trackZd = otio.adapters.read_from_string("""
        {
            "OTIO_SCHEMA": "Track.1",
            "children": [
                {
                    "OTIO_SCHEMA": "Clip.1",
                    "effects": [],
                    "markers": [],
                    "media_reference": null,
                    "enabled": false,
                    "metadata": {},
                    "name": "Z",
                    "source_range": {
                        "OTIO_SCHEMA": "TimeRange.1",
                        "duration": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 150
                        },
                        "start_time": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 0.0
                        }
                    }
                }
            ],
            "effects": [],
            "kind": "Video",
            "markers": [],
            "metadata": {},
            "name": "Sequence1",
            "source_range": null
        }
        """, "otio_json")

        self.track_d = otio.adapters.read_from_string("""
        {
            "OTIO_SCHEMA": "Track.1",
            "children": [
                {
                    "OTIO_SCHEMA": "Clip.1",
                    "effects": [],
                    "markers": [],
                    "media_reference": null,
                    "enabled": true,
                    "metadata": {},
                    "name": "Z",
                    "source_range": {
                        "OTIO_SCHEMA": "TimeRange.1",
                        "duration": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 150
                        },
                        "start_time": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 0.0
                        }
                    }
                }
            ],
            "effects": [],
            "kind": "Video",
            "markers": [],
            "enabled": false,
            "metadata": {},
            "name": "Sequence1",
            "source_range": null
        }
        """, "otio_json")

        self.trackABC = otio.adapters.read_from_string("""
        {
            "OTIO_SCHEMA": "Track.1",
            "children": [
                {
                    "OTIO_SCHEMA": "Clip.1",
                    "effects": [],
                    "markers": [],
                    "media_reference": null,
                    "metadata": {},
                    "name": "A",
                    "source_range": {
                        "OTIO_SCHEMA": "TimeRange.1",
                        "duration": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 50
                        },
                        "start_time": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 0.0
                        }
                    }
                },
                {
                    "OTIO_SCHEMA": "Clip.1",
                    "effects": [],
                    "markers": [],
                    "media_reference": null,
                    "metadata": {},
                    "name": "B",
                    "source_range": {
                        "OTIO_SCHEMA": "TimeRange.1",
                        "duration": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 50
                        },
                        "start_time": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 0.0
                        }
                    }
                },
                {
                    "OTIO_SCHEMA": "Clip.1",
                    "effects": [],
                    "markers": [],
                    "media_reference": null,
                    "metadata": {},
                    "name": "C",
                    "source_range": {
                        "OTIO_SCHEMA": "TimeRange.1",
                        "duration": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 50
                        },
                        "start_time": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 0.0
                        }
                    }
                }
            ],
            "effects": [],
            "kind": "Video",
            "markers": [],
            "metadata": {},
            "name": "Sequence1",
            "source_range": null
        }
        """, "otio_json")

        self.trackDgE = otio.adapters.read_from_string("""
        {
            "OTIO_SCHEMA": "Track.1",
            "children": [
                {
                    "OTIO_SCHEMA": "Clip.1",
                    "effects": [],
                    "markers": [],
                    "media_reference": null,
                    "metadata": {},
                    "name": "D",
                    "source_range": {
                        "OTIO_SCHEMA": "TimeRange.1",
                        "duration": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 50
                        },
                        "start_time": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 0.0
                        }
                    }
                },
                {
                    "OTIO_SCHEMA": "Gap.1",
                    "effects": [],
                    "markers": [],
                    "media_reference": null,
                    "metadata": {},
                    "name": "g",
                    "source_range": {
                        "OTIO_SCHEMA": "TimeRange.1",
                        "duration": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 50
                        },
                        "start_time": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 0.0
                        }
                    }
                },
                {
                    "OTIO_SCHEMA": "Clip.1",
                    "effects": [],
                    "markers": [],
                    "media_reference": null,
                    "metadata": {},
                    "name": "E",
                    "source_range": {
                        "OTIO_SCHEMA": "TimeRange.1",
                        "duration": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 50
                        },
                        "start_time": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 0.0
                        }
                    }
                }
            ],
            "effects": [],
            "kind": "Video",
            "markers": [],
            "metadata": {},
            "name": "Sequence1",
            "source_range": null
        }
        """, "otio_json")

        self.trackgFg = otio.adapters.read_from_string("""
        {
            "OTIO_SCHEMA": "Track.1",
            "children": [
                {
                    "OTIO_SCHEMA": "Gap.1",
                    "effects": [],
                    "markers": [],
                    "media_reference": null,
                    "metadata": {},
                    "name": "g1",
                    "source_range": {
                        "OTIO_SCHEMA": "TimeRange.1",
                        "duration": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 50
                        },
                        "start_time": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 0.0
                        }
                    }
                },
                {
                    "OTIO_SCHEMA": "Clip.1",
                    "effects": [],
                    "markers": [],
                    "media_reference": null,
                    "metadata": {},
                    "name": "F",
                    "source_range": {
                        "OTIO_SCHEMA": "TimeRange.1",
                        "duration": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 50
                        },
                        "start_time": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 0.0
                        }
                    }
                },
                {
                    "OTIO_SCHEMA": "Gap.1",
                    "effects": [],
                    "markers": [],
                    "media_reference": null,
                    "metadata": {},
                    "name": "g2",
                    "source_range": {
                        "OTIO_SCHEMA": "TimeRange.1",
                        "duration": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 50
                        },
                        "start_time": {
                            "OTIO_SCHEMA": "RationalTime.1",
                            "rate": 24,
                            "value": 0.0
                        }
                    }
                }
            ],
            "effects": [],
            "kind": "Video",
            "markers": [],
            "metadata": {},
            "name": "Sequence1",
            "source_range": null
        }
        """, "otio_json")

    def test_flatten_single_track(self):
        stack = otio.schema.Stack(children=[
            self.trackABC
        ])
        flat_track = otio.algorithms.flatten_stack(stack)
        # the result should be equivalent
        self.assertJsonEqual(
            flat_track[:],
            self.trackABC[:]
        )
        # but not the same actual objects
        self.assertIsNot(
            flat_track[0],
            self.trackABC[0]
        )
        self.assertIsNot(
            flat_track[1],
            self.trackABC[1]
        )
        self.assertIsNot(
            flat_track[2],
            self.trackABC[2]
        )

    def test_flatten_obscured_track(self):
        stack = otio.schema.Stack(children=[
            self.trackABC,
            self.trackZ
        ])
        flat_track = otio.algorithms.flatten_stack(stack)
        self.assertJsonEqual(
            flat_track[:],
            self.trackZ[:]
        )

        # It is an error to add an item to composition if it is already in
        # another composition.  This clears out the old test composition
        # (and also clears out its parent pointers).
        del stack
        stack = otio.schema.Stack(
            children=[
                self.trackZ,
                self.trackABC,
            ]
        )
        flat_track = otio.algorithms.flatten_stack(stack)
        self.assertJsonEqual(
            flat_track[:],
            self.trackABC[:]
        )

    def test_flatten_disabled_clip(self):
        stack = otio.schema.Stack(children=[
            self.trackABC,
            self.trackZ
        ])
        flat_track = otio.algorithms.flatten_stack(stack)
        self.assertJsonEqual(
            flat_track[:],
            self.trackZ[:]
        )
        del stack
        stack = otio.schema.Stack(children=[
            self.trackABC,
            self.trackZd
        ])
        flat_track = otio.algorithms.flatten_stack(stack)
        self.assertJsonEqual(
            flat_track[:],
            self.trackABC[:]
        )

    def test_flatten_disabled_track(self):
        stack = otio.schema.Stack(children=[
            self.trackABC,
            self.trackZ
        ])
        flat_track = otio.algorithms.flatten_stack(stack)
        self.assertJsonEqual(
            flat_track[:],
            self.trackZ[:]
        )
        del stack
        stack = otio.schema.Stack(children=[
            self.trackABC,
            self.track_d
        ])
        flat_track = otio.algorithms.flatten_stack(stack)
        self.assertJsonEqual(
            flat_track[:],
            self.trackABC[:]
        )

    def test_flatten_gaps(self):
        stack = otio.schema.Stack(children=[
            self.trackABC,
            self.trackDgE
        ])
        flat_track = otio.algorithms.flatten_stack(stack)
        self.assertIsOTIOEquivalentTo(flat_track[0], self.trackDgE[0])
        self.assertIsOTIOEquivalentTo(flat_track[1], self.trackABC[1])
        self.assertIsOTIOEquivalentTo(flat_track[2], self.trackDgE[2])
        self.assertIsNot(flat_track[0], self.trackDgE[0])
        self.assertIsNot(flat_track[1], self.trackABC[1])
        self.assertIsNot(flat_track[2], self.trackDgE[2])

        # create a new stack out of the old parts, delete the old stack first
        del stack
        stack = otio.schema.Stack(children=[
            self.trackABC,
            self.trackgFg
        ])
        flat_track = otio.algorithms.flatten_stack(stack)
        self.assertIsOTIOEquivalentTo(flat_track[0], self.trackABC[0])
        self.assertIsOTIOEquivalentTo(flat_track[1], self.trackgFg[1])
        self.assertIsOTIOEquivalentTo(flat_track[2], self.trackABC[2])
        self.assertIsNot(flat_track[0], self.trackABC[0])
        self.assertIsNot(flat_track[1], self.trackgFg[1])
        self.assertIsNot(flat_track[2], self.trackABC[2])

    def test_flatten_gaps_with_trims(self):
        stack = otio.schema.Stack(children=[
            self.trackZ,
            self.trackDgE
        ])
        flat_track = otio.algorithms.flatten_stack(stack)
        self.assertIsOTIOEquivalentTo(flat_track[0], self.trackDgE[0])
        self.assertEqual(flat_track[1].name, "Z")
        self.assertEqual(
            flat_track[1].source_range,
            otio.opentime.TimeRange(
                otio.opentime.RationalTime(50, 24),
                otio.opentime.RationalTime(50, 24)
            )
        )
        self.assertIsOTIOEquivalentTo(flat_track[2], self.trackDgE[2])

        del stack
        stack = otio.schema.Stack(children=[
            self.trackZ,
            self.trackgFg
        ])
        flat_track = otio.algorithms.flatten_stack(stack)
        self.assertEqual(flat_track[0].name, "Z")
        self.assertEqual(
            flat_track[0].source_range,
            otio.opentime.TimeRange(
                otio.opentime.RationalTime(0, 24),
                otio.opentime.RationalTime(50, 24)
            )
        )
        self.assertIsOTIOEquivalentTo(flat_track[1], self.trackgFg[1])
        self.assertEqual(flat_track[2].name, "Z")
        self.assertEqual(
            flat_track[2].source_range,
            otio.opentime.TimeRange(
                otio.opentime.RationalTime(100, 24),
                otio.opentime.RationalTime(50, 24)
            )
        )

    def test_flatten_list_of_tracks(self):
        tracks = [
            self.trackABC,
            self.trackDgE
        ]
        flat_track = otio.algorithms.flatten_stack(tracks)
        self.assertIsOTIOEquivalentTo(flat_track[0], self.trackDgE[0])
        self.assertIsOTIOEquivalentTo(flat_track[1], self.trackABC[1])
        self.assertIsOTIOEquivalentTo(flat_track[2], self.trackDgE[2])

        tracks = [
            self.trackABC,
            self.trackgFg
        ]
        flat_track = otio.algorithms.flatten_stack(tracks)
        self.assertIsOTIOEquivalentTo(flat_track[0], self.trackABC[0])
        self.assertIsOTIOEquivalentTo(flat_track[1], self.trackgFg[1])
        self.assertIsOTIOEquivalentTo(flat_track[2], self.trackABC[2])

    def test_flatten_example_code(self):
        timeline = otio.adapters.read_from_file(MULTITRACK_EXAMPLE_PATH)
        preflattened = otio.adapters.read_from_file(PREFLATTENED_EXAMPLE_PATH)
        preflattened_track = preflattened.video_tracks()[0]
        flattened_track = otio.algorithms.flatten_stack(
            timeline.video_tracks()
        )

        # the names will be different, so clear them both
        preflattened_track.name = ""
        flattened_track.name = ""

        self.assertOTIOEqual(
            preflattened_track,
            flattened_track
        )

    def assertOTIOEqual(self, a, b):
        self.maxDiff = None
        self.assertMultiLineEqual(
            otio.adapters.write_to_string(a, 'otio_json'),
            otio.adapters.write_to_string(b, 'otio_json')
        )

    def test_flatten_with_transition(self):
        stack = otio.schema.Stack(
            children=[
                self.trackABC,
                self.trackDgE,
            ]
        )
        stack[1].insert(
            1,
            otio.schema.Transition(
                name="test_transition",
                in_offset=otio.opentime.RationalTime(10, 24),
                out_offset=otio.opentime.RationalTime(15, 24)
            )
        )
        flat_track = otio.algorithms.flatten_stack(stack)
        self.assertEqual(3, len(self.trackABC))
        self.assertEqual(4, len(stack[1]))
        self.assertEqual(4, len(flat_track))
        self.assertEqual(flat_track[1].name, "test_transition")

    def test_top_child_at_time(self):
        stack = otio.schema.Stack(
            children=[
                self.trackABC,
                self.trackDgE,
            ]
        )

        top_child = otio.algorithms.top_clip_at_time(
            stack,
            otio.opentime.RationalTime(0, 24)
        )
        self.assertEqual(top_child, self.trackDgE[0])

        stack.append(
            otio.schema.Track(
                children=[
                    otio.schema.Gap(
                        source_range=otio.opentime.TimeRange(
                            otio.opentime.RationalTime(0, 24),
                            otio.opentime.RationalTime(10, 24)
                        )
                    )
                ]
            )
        )

        top_child = otio.algorithms.top_clip_at_time(
            stack,
            otio.opentime.RationalTime(0, 24)
        )
        self.assertEqual(top_child, self.trackDgE[0])


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import unittest
import os

import opentimelineio as otio
from tests import baseline_reader

"""Unit tests for the schemadef plugin system."""


SCHEMADEF_NAME = "schemadef_example"
EXAMPLE_ARG = "exampleArg"
EXCLASS = "<class 'opentimelineio.schemadef.example_schemadef.exampleSchemaDef'>"
TEST_STRING = """
{
    "OTIO_SCHEMA": "exampleSchemaDef.1",
    "exampleArg": "foobar"
}
"""


def _clean_plugin_module():
    """Remove the example_schemadef if its already been loaded to test
    autoload/explicit load behavior.
    """
    try:
        del otio.schemadef.example_schemadef
    except AttributeError:
        pass

    try:
        plugin = otio.schema.schemadef.from_name("example_schemadef")
        plugin._module = None
    except otio.exceptions.NotSupportedError:
        pass


class TestPluginSchemadefs(unittest.TestCase):
    def setUp(self):
        self.save_manifest = otio.plugins.manifest._MANIFEST
        self.save_manifest_path = os.environ.get('OTIO_PLUGIN_MANIFEST_PATH')
        # find the path to the baselines/schemadef_example.json
        self.manifest_path = baseline_reader.path_to_baseline(SCHEMADEF_NAME)
        os.environ['OTIO_PLUGIN_MANIFEST_PATH'] = self.manifest_path
        otio.plugins.manifest.ActiveManifest(force_reload=True)
        _clean_plugin_module()

    def tearDown(self):
        # restore original state
        if self.save_manifest_path:
            os.environ['OTIO_PLUGIN_MANIFEST_PATH'] = self.save_manifest_path
        else:
            del os.environ['OTIO_PLUGIN_MANIFEST_PATH']
        otio.plugins.manifest._MANIFEST = self.save_manifest

        _clean_plugin_module()

    def test_autoloaded_plugin(self):
        with self.assertRaises(AttributeError):
            otio.schemadef.example_schemadef
        # should force an autoload
        thing = otio.adapters.read_from_string(TEST_STRING, "otio_json")
        self.assertEqual(thing.exampleArg, "foobar")

    def test_plugin_schemadef(self):
        with self.assertRaises(AttributeError):
            otio.schemadef.example_schemadef
        # force loading the module
        otio.schema.schemadef.module_from_name("example_schemadef")

        # Our test manifest should have been loaded, including
        # the example_schemadef.
        # Try creating a schema object using the instance_from_schema method.
        peculiar_value = "something One-derful"
        example = otio.core.instance_from_schema("exampleSchemaDef", 1, {
            EXAMPLE_ARG: peculiar_value
        })

        self.assertEqual(str(type(example)), EXCLASS)
        self.assertEqual(example.exampleArg, peculiar_value)

    def test_plugin_schemadef_namespace(self):
        with self.assertRaises(AttributeError):
            otio.schemadef.example_schemadef

        # force loading the module
        plugin_module = otio.schema.schemadef.module_from_name(
            "example_schemadef"
        )

        # Try creating schema object with the direct class definition method:
        peculiar_value = "something Two-derful"
        example = otio.schemadef.example_schemadef.exampleSchemaDef(peculiar_value)
        self.assertEqual(plugin_module, otio.schemadef.example_schemadef)
        self.assertEqual(str(type(example)), EXCLASS)
        self.assertEqual(example.exampleArg, peculiar_value)


if __name__ == '__main__':
    unittest.main()
